/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_ALG_TEST_TRAVERSE_VISITOR__
#define __DD_ALG_TEST_TRAVERSE_VISITOR__

#include "DDTreeNode.h"
#include "DDSingleSetTraverseVisitor.h"
#include "DDRankCalculator.h"
#include "DDSingleSetRay.h"
#include "DDAlgTestSingleSetRayCacheEntry.h"

namespace addibit {

    class AlgTestTraverseVisitor : public SingleSetTraverseVisitor<AlgTestSingleSetRayCacheEntry> {

        private:
        const Matrix & mSystem;
        RankCalculator mCalculator;
        
        virtual void create_shuffled ( const SingleSetRay * const, const SingleSetRay * const ) = 0;

        public:
        AlgTestTraverseVisitor( const TreeNode * const tree,
                                const RayWrapperContainer<SingleSetRay> & container,
                                const uint_fast16_t stack_size,
                                const uint_fast16_t plane,
                                const Matrix & system,
                                CacheEntryNode<SingleSetRay, AlgTestSingleSetRayCacheEntry> & node ) :
            SingleSetTraverseVisitor<AlgTestSingleSetRayCacheEntry>( tree, container, stack_size, plane, node ),
            mSystem( system ),
            mCalculator( system.getRows(), system.getColumns() ) {}
        
        void verify_edge( const SingleSetRay * const ray, const AdjacencyCandidate * const candidate ) {
            const SingleSetRay * const r_2 = reinterpret_cast<const SingleSetRay *>( candidate->id() );
            
            if ( are_trivially_adjacent( ray, r_2, candidate ) ) {
                create_shuffled( ray, r_2 );
            } else {                
                mCalculator.load ( mSystem, candidate->data() );      
                
                if ( mCalculator.hasMinimumRank ( GLOBAL_EDGE_DIM ) ) {
                    create_shuffled( ray, r_2 );
                }
            }            
        }
        
        using PreliminaryTraverseVisitor<SingleSetRay, AlgTestSingleSetRayCacheEntry>::mCandidatesContainer;
        using SingleSetTraverseVisitor<AlgTestSingleSetRayCacheEntry>::narrow;

        inline void process_ray ( const SingleSetRay & ray ) {
            narrow( ray.getActiveSet() );
            mCandidatesContainer.verify<SingleSetRay, AlgTestTraverseVisitor>( &ray, *this );
        } 
    };
    
    class PositiveAlgTestTraverseVisitor : public AlgTestTraverseVisitor {
        
        protected:
        using SingleSetTraverseVisitor::create;
        
        void create_shuffled ( const SingleSetRay * const negative, const SingleSetRay * const positive ) {
            create( positive, negative );            
        }
        
        public:
        PositiveAlgTestTraverseVisitor( const TreeNode * const tree,
                                        const RayWrapperContainer<SingleSetRay> & container,
                                        const uint_fast16_t stack_size,
                                        const uint_fast16_t plane,
                                        const Matrix & system,
                                        CacheEntryNode<SingleSetRay, AlgTestSingleSetRayCacheEntry> & node ) :
            AlgTestTraverseVisitor( tree, container, stack_size, plane, system, node ) {}
    };

    class NegativeAlgTestTraverseVisitor : public AlgTestTraverseVisitor {
        
        protected:
        using SingleSetTraverseVisitor::create;
        
        void create_shuffled ( const SingleSetRay * const positive, const SingleSetRay * const negative ) {            
            create( positive, negative );            
        }
        
        public:
        NegativeAlgTestTraverseVisitor( const TreeNode * const tree,
                                        const RayWrapperContainer<SingleSetRay> & container,
                                        const uint_fast16_t stack_size,
                                        const uint_fast16_t plane,
                                        const Matrix & system,
                                        CacheEntryNode<SingleSetRay, AlgTestSingleSetRayCacheEntry> & node ) :
            AlgTestTraverseVisitor( tree, container, stack_size, plane, system, node ) {}
    };    
}

#endif // __DD_ALG_TEST_TRAVERSE_VISITOR__
