/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BIT_LIST_ARRAY__
#define __DD_BIT_LIST_ARRAY__

#include "DDTweaks.h"
#include "DDBitList.h"

#include "DDTreeNodeCondition.h"
#include "DDNarrowingCandidate.h"

namespace addibit {

	class BitListArray {

        private:
        const int_fast32_t mEntries;
        const uint_fast32_t mChunks;

        __BIT_LIST_RAW_DATA_TYPE__ * mData;
        __BIT_LIST_RAW_DATA_TYPE__ * __region_end;

        __BIT_LIST_RAW_DATA_TYPE__ * alloc_space ( const uint_fast32_t chunks ) {
            return new __BIT_LIST_RAW_DATA_TYPE__[ chunks ];
        }

        bool equals ( const unsigned long lhs, const unsigned long rhs ) const {
            return rhs == lhs;
        }

        bool equals ( const unsigned long * lhs, const BitList & rhs ) const {
            bool result = equals( lhs[0], rhs[0] );

            for ( uint_fast16_t block = 1; result && block < __DD_BLOCK_COUNT__ ; block++ ) {
                result = result && equals( lhs[ block ], rhs[ block ] );
            }

            return result;
        }

        public:
        BitListArray () :
            mEntries( 0 ),
            mChunks( 0 ),
            mData( NULL ),
            __region_end( NULL ) {}        
        
        BitListArray ( const uint_fast32_t entries ) :
            mEntries( entries ),
            mChunks( entries * __DD_BLOCK_COUNT__  ),
            mData( alloc_space( mChunks ) ),
            __region_end( mData + mChunks ) {}
        
        BitListArray ( const uint_fast32_t entries,
                       char * memory ) :
            mEntries( entries ),
            mChunks( entries * __DD_BLOCK_COUNT__  ),
            mData( reinterpret_cast<__BIT_LIST_RAW_DATA_TYPE__ *>( memory ) ),
            __region_end( mData + mChunks ) {}

     /*   bool contains ( const BitList & list, const BitList & illegal_1, const BitList & illegal_2 ) const {
        
            for ( unsigned long * lhs = mData; lhs < mData + mChunks; lhs += __DD_BLOCK_COUNT__  ) {
                if ( contains( lhs, list ) && !equals( lhs, illegal_1 ) && !equals( lhs, illegal_2 ) ) {
                    return true;
                }
            }

            return false;
        } */
        
        inline int_fast32_t contains ( const BitList & __cand, const uint_fast32_t starting_entry = 0 ) const {
            return contains( __cand.data(), starting_entry );
        }
        
        int_fast32_t contains ( const __BIT_LIST_RAW_DATA_TYPE__ * const __cand, const int_fast32_t starting_entry = 0 ) const {
        
            for ( __BIT_LIST_RAW_DATA_TYPE__ * __lhs = mData + starting_entry * __DD_BLOCK_COUNT__; __lhs < mData + mChunks; __lhs += __DD_BLOCK_COUNT__  ) {
    #ifdef __STATS_OUTPUT__
            TreeNodeCondition::VERIFICATION_OPS++;
    #endif                
                if ( BitList::a_contains_b( __lhs, __cand ) ) {
                    return ( __lhs - mData ) / __DD_BLOCK_COUNT__;
                }
            }
            return entries();
        }
        
        inline __BIT_LIST_RAW_DATA_TYPE__ * data_begin () const {
            return mData;
        }
        
        inline __BIT_LIST_RAW_DATA_TYPE__ * data_end () const {
            return __region_end;
        }
        
        inline bool next_feasible ( const NarrowingCandidate & cand, const __BIT_LIST_RAW_DATA_TYPE__ * & __entry ) const {
            return next_feasible( &cand, __entry );
        }
        
        bool next_feasible ( const NarrowingCandidate * const cand, const __BIT_LIST_RAW_DATA_TYPE__ * & __entry ) const {
            
            for ( ; __entry != __region_end; __entry += __DD_BLOCK_COUNT__ ) {
                
    #ifdef __STATS_OUTPUT__
                TreeNodeCondition::NARROWING_OPS++;
    #endif
                if ( BitList::getCommonPopCnt( cand->data(), __entry ) >= GLOBAL_EDGE_DIM ) {
                    return true;
                }
            }
            return false;
        }

        inline uint_fast16_t getCommonPopCount ( const uint_fast32_t index, const BitList & list ) const {
            return list.getCommonPopCnt( mData + ( index * __DD_BLOCK_COUNT__  ) );
        }

        const __BIT_LIST_RAW_DATA_TYPE__ * operator[] ( const uint_fast32_t index ) const {
            return mData + ( __DD_BLOCK_COUNT__  * index );
        }

        BitList entry ( const uint_fast32_t index ) const {
            BitList result( __BIT_LIST_ALLOC_SPACE__ );
            unsigned long * offset = mData + ( __DD_BLOCK_COUNT__  * index );

            result[ 0 ] = offset[ 0 ];

            for ( uint_fast16_t block = 1; block < __DD_BLOCK_COUNT__ ; block++ ) {
                result[ block ] = offset[ block ];
            }

            return result;
        }
        
        const __BIT_LIST_RAW_DATA_TYPE__ * raw_entry ( const uint_fast32_t index ) const {
            return mData + ( __DD_BLOCK_COUNT__  * index );
        }

        void set ( const uint_fast32_t index, const BitList & list ) {
            __BIT_LIST_RAW_DATA_TYPE__ * offset = mData + ( __DD_BLOCK_COUNT__  * index );

            DD_LOG_DEBUG( "Inserting new vector with " << list.getPopulationCount() << " bits." )

            offset[ 0 ] = list[ 0 ];

            for ( uint_fast16_t block = 1; block < __DD_BLOCK_COUNT__ ; block++ ) {
                offset[ block ] = list[ block ];
            }        
        }
        
        void set ( const uint_fast32_t index, const BitList & list, const uint_fast32_t aux ) {
            __BIT_LIST_RAW_DATA_TYPE__ * offset = mData + ( ( __DD_BLOCK_COUNT__ + 1 ) * index );

            DD_LOG_DEBUG( "Inserting new vector with " << list.getPopulationCount() << " bits." )

            offset[ 0 ] = list[ 0 ];

            for ( uint_fast16_t block = 1; block < __DD_BLOCK_COUNT__ ; block++ ) {
                offset[ block ] = list[ block ];
            }  
            
            offset[ __DD_BLOCK_COUNT__ ] = aux;      
        }

        inline int_fast32_t entries () const {
            return mEntries;
        }

        inline bool empty () const {
            return mData == NULL;
        }
        
        const std::string toString () const;
    };
}

#endif // __DD_BIT_LIST_ARRAY__
