
#ifndef __DD_BIT_SET_TREE_NODE__
#define __DD_BIT_SET_TREE_NODE__

#include <limits.h>
#include <vector>

#include "DDBitList.h"

namespace addibit {

	template <class RAY_TYPE>
	class BitSetTreeNode {
		
		private:
		BitList mMap;
		BitList mUnpartitionedBits;
		BitSetTreeNode* mSuperSet;
		BitSetTreeNode* mSubSet;

		std::vector<const RAY_TYPE*> mRays;

		public:
		BitSetTreeNode( const unsigned char blocks ) :
			mMap( blocks, 0 ),
			mUnpartitionedBits( blocks, ULONG_MAX ),
			mSuperSet( NULL ),
			mSubSet( NULL ) {};

		BitSetTreeNode( const unsigned char blocks, const unsigned long pattern ) :
			mMap( blocks, 0 ),
			mUnpartitionedBits( blocks, pattern ),
			mSuperSet( NULL ),
			mSubSet( NULL ) {};

		BitSetTreeNode( const BitList & unpartitioned ) :
			mMap( unpartitioned.getBlocks(), 0 ),
			mUnpartitionedBits( unpartitioned ),
			mSuperSet( NULL ),
			mSubSet( NULL ) {};

		BitSetTreeNode* getSuperSet () const {
			return mSuperSet;
		}

		BitSetTreeNode* getSubSet () const {
			return mSubSet;
		}

		const BitList & getMap () const {
			return mMap;
		}

		const std::vector<const RAY_TYPE*> & getRays () const {
			return mRays;
		}

		bool isLeaf () const {
			return mSuperSet == NULL;
		}

		void insert ( const RAY_TYPE* const ray ) {

			mMap |= ray->getActiveSet();
			mRays.push_back( ray );	
		}

		void partition ( const unsigned int threshold ) {

			const BitList candidates = mMap & mUnpartitionedBits;

			if ( mRays.size() > threshold && !candidates.isEmpty() ) {
				const unsigned long partition_bit = candidates.getFirstSet();

				mUnpartitionedBits.unsetBit( partition_bit );

				mSuperSet = new BitSetTreeNode( mUnpartitionedBits );
				mSubSet = new BitSetTreeNode( mUnpartitionedBits );
		
				for ( unsigned int index = 0; index < mRays.size(); index++ ) {
					if ( mRays[index]->getActiveSet().isSet( partition_bit ) ) {
						mSuperSet->insert( mRays[index] );
					} else {
						mSubSet->insert( mRays[index] );
					}
				}

				if ( mSubSet->getRays().size() < threshold/5 ) {
					delete mSuperSet;
					delete mSubSet;

					partition( threshold );
				} else {
					mRays.clear();

					mSuperSet->partition( threshold );
					mSubSet->partition( threshold );
				}
			} else {
				mSuperSet = NULL;
				mSubSet = NULL;
			} 
		}
	};
}

#endif // __DD_BIT_SET_TREE_NODE__
