/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_CACHE_ENTRY__
#define __DD_CACHE_ENTRY__

#include <vector>
#include <sstream>
#include <algorithm>
#include <math.h>

#include "DDStats.h"

#include "DDOptions.h"

#include "DDBitList.h"
#include "DDBitCountTable.h"
#include "DDBitSelector.h"

#include "DDRayWrapperContainer.h"
#include "DDRayRefReleaseVisitor.h"

#include "DDBitCountVisitor.h"
#include "DDBitFrequencyModifier.h"
#include "DDZeroFrequencyModifier.h"

#include "DDNarrowQueryBitsNeutralizationStrategy.h"
#include "DDMostAverageBitFirstStrategy.h"
#include "DDLeastCommonBitFirstStrategy.h"
#include "DDMostCommonBitFirstStrategy.h"

#include "DDDegeneracyCalcVisitor.h"

namespace addibit {
    
    template <class RAY_TYPE>
    class DummyCacheEntry {
        
        public:
        bool proceed ( std::shared_ptr<const RAY_TYPE > ) {
            return true;
        }
    };
	
	template <class RAY_TYPE>
	class CacheEntry {       

		protected:
        const uint_fast16_t mRows;
		const uint_fast16_t mPlane;
		const uint_fast16_t mPlaneCoeffOffset;
		const uint_fast16_t mBlocks;
        
        const BitList mPattern;
        
        const uint_fast16_t mNarDegThreshold;

		RayWrapperContainer<RAY_TYPE> mPositiveEntriesHighDeg;
		RayWrapperContainer<RAY_TYPE> mPositiveEntriesLowDeg;
		
		RayWrapperContainer<RAY_TYPE> mNegativeEntriesHighDeg;
		RayWrapperContainer<RAY_TYPE> mNegativeEntriesLowDeg;                   

        inline uint_fast16_t ray_dim () const {
            return GLOBAL_EDGE_DIM + 1;
        }

   /*     void calcFrequency ( RayWrapperContainer<RAY_TYPE> & container,
                             const RayWrapperContainer<RAY_TYPE> & query_data,
                             const bool value = 1 ) const {

            BitCountVisitor<RAY_TYPE> bit_counter( mPlane );
            query_data.iterate( bit_counter );
            if ( value ) {
                const BitFrequencyModifier modifier( bit_counter.population());
                container.modify( modifier );
            } else {
                const ZeroFrequencyModifier modifier( bit_counter.population(), container.size() );
                container.modify( modifier );
            }
        }

        void sort_data ( RayWrapperContainer<RAY_TYPE> & container, RayWrapperContainer<RAY_TYPE> & query_data, const SortType & type ) {
            
            switch ( type ) {
                case MAX_SET_BITS:
                    container.template sort<RefComparatorMostBitsFirst>( RefComparatorMostBitsFirst() );
                    break;
                case MIN_SET_BITS:
                    container.template sort<RefComparatorLeastBitsFirst>( RefComparatorLeastBitsFirst() );
                    break;
                case MAX_SET_BITS_LEX:
                    container.template sort<RefComparatorMostBitsFirstLex>( RefComparatorMostBitsFirstLex() );
                    break;
                case MIN_SET_BITS_LEX:
                    container.template sort<RefComparatorLeastBitsFirstLex>( RefComparatorLeastBitsFirstLex() );
                    break;
                case MAX_BIT_FREQUENCY:
                    calcFrequency( container, query_data );
                    container.template sort<RefComparatorMaxFrequency>( RefComparatorMaxFrequency() );
                    break;
                case MIN_BIT_FREQUENCY:
                   calcFrequency( container, query_data );
                    container.template sort<RefComparatorMinFrequency>( RefComparatorMinFrequency() );
                    break;
                case MAX_SET_BITS_MAX_BIT_FREQUENCY:
                   calcFrequency( container, query_data );
                    container.template sort<RefComparatorMaxSetBitsMaxFrequency>( RefComparatorMaxSetBitsMaxFrequency() );
                    break;   
                case MAX_SET_BITS_MIN_BIT_FREQUENCY:
                   calcFrequency( container, query_data );
                    container.template sort<RefComparatorMaxSetBitsMinFrequency>( RefComparatorMaxSetBitsMinFrequency() );
                    break;       
                case MIN_SET_BITS_MIN_BIT_FREQUENCY:
                   calcFrequency( container, query_data );
                    container.template sort<RefComparatorMinSetBitsMinFrequency>( RefComparatorMinSetBitsMinFrequency() );
                    break; 
                case MIN_SET_BITS_MAX_BIT_FREQUENCY:
                   calcFrequency( container, query_data );
                    container.template sort<RefComparatorMinSetBitsMaxFrequency>( RefComparatorMinSetBitsMaxFrequency() );
                    break; 
                default:
                    // Do nothing here    
            }
        } */
        
        const BitList create_cut_map () const {
            BitList map( mBlocks, 0 );
            map.setBits( 0, mPlane );
            return map;
        }   
        
        inline bool insert_narrow ( std::shared_ptr<const RAY_TYPE> ray ) {
                                        
            const uint_fast16_t deg = mPattern.getCommonPopCnt( ray->getActiveSet() );
            bool result = false;

            if ( deg > mNarDegThreshold && ray->isInside( plane() ) ) {
                mPositiveEntriesHighDeg.insert( ray );
                result = true;
            } else if ( deg > mNarDegThreshold ) {
                mNegativeEntriesHighDeg.insert( ray );
            } else if ( ray->isInside( plane() ) ) {
                mPositiveEntriesLowDeg.insert( ray );
                result = true;
            } else {
                mNegativeEntriesLowDeg.insert( ray );
            }              
            
            return result;
        }        

		public:            
		CacheEntry ( const uint_fast16_t rows,
                     const uint_fast16_t current_plane,
                     const BitList & pattern ) :
            mRows( rows ),
			mPlane( current_plane ),
			mPlaneCoeffOffset( current_plane + 1 ),
			mBlocks( BitList::getBlocks( rows ) ),
            mPattern( pattern ),
            mNarDegThreshold( Options::current().narrowing_degeneracy_threshold() + GLOBAL_EDGE_DIM ) {} 
            
		CacheEntry ( const uint_fast16_t rows,
                     const uint_fast16_t current_plane,
                     const uint_fast16_t plane_coeff_offset,
                     const BitList & pattern ) :
            mRows( rows ),
			mPlane( current_plane ),
			mPlaneCoeffOffset( plane_coeff_offset ),
			mBlocks( BitList::getBlocks( rows ) ),
            mPattern( pattern ),
            mNarDegThreshold( Options::current().narrowing_degeneracy_threshold() + GLOBAL_EDGE_DIM ) {}                          

        virtual ~CacheEntry () {}   

        const RayWrapperContainer<RAY_TYPE> & positive_hd () const {
            return mPositiveEntriesHighDeg;
        }
        
        const RayWrapperContainer<RAY_TYPE> & positive_ld () const {
            return mPositiveEntriesLowDeg;
        }        

        const RayWrapperContainer<RAY_TYPE> & negative_hd () const {
            return mNegativeEntriesHighDeg;
        }
        
        const RayWrapperContainer<RAY_TYPE> & negative_ld () const {
            return mNegativeEntriesLowDeg;
        }              	

		uint_fast32_t getPositiveCount () const {
			return mPositiveEntriesHighDeg.size() + mPositiveEntriesLowDeg.size();
		}

		uint_fast32_t getNegativeCount () const {
			return mNegativeEntriesHighDeg.size() + mNegativeEntriesLowDeg.size();
		}
		
		uint_fast16_t plane () const {
		    return mPlane;
		}
		
		uint_fast16_t plane_coeff_offset () const {
		    return mPlaneCoeffOffset;
		}		
        
        inline const BitList & pattern () const {
            return mPattern;
        }
        
        double_t degeneracy () const {
            DegeneracyCalcVisitor<RAY_TYPE> deg_visitor( pattern() );
            
            mPositiveEntriesHighDeg.template iterate<DegeneracyCalcVisitor<RAY_TYPE> >( deg_visitor );
            mPositiveEntriesLowDeg.template iterate<DegeneracyCalcVisitor<RAY_TYPE> >( deg_visitor );
            mNegativeEntriesHighDeg.template iterate<DegeneracyCalcVisitor<RAY_TYPE> >( deg_visitor );
            mNegativeEntriesLowDeg.template iterate<DegeneracyCalcVisitor<RAY_TYPE> >( deg_visitor ); 
            
            return deg_visitor.degeneracy();
        }
		
		virtual bool proceed ( std::shared_ptr<const RAY_TYPE > ) = 0;		
        
        void prepare_for_test () {
           /* const Options & options = Options::current();
                        
            sort_data( mPositiveEntriesHighDeg, mNegativeEntriesHighDeg, options.narrowing().data_sort() ); */
        }
        
        virtual const std::string toString () const {
            std::stringstream result;
            
            result << "plane=" << plane() << ", offset=" << plane_coeff_offset();
            
            return result.str();
        } 
	};
}

#endif // __DD_CACHE_ENTRY__
