/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_CALC_MODULE__
#define __DD_CALC_MODULE__

#include <cstring>
#include <gmp.h>
#include <gmpxx.h>

#include "DDTweaks.h"

namespace addibit {

	class CalcModule {

		private:
		mpz_class register_x;
		mpz_class register_y;

		mpz_class register_s;
		mpz_class register_s1;
		mpz_class register_s2;

		mpz_class register_t;
		mpz_class register_t1;
		mpz_class register_t2;

		mpz_class register_g;
		mpz_class register_r;

		mpz_class register_p;
		mpz_class register_q;

		public:

		inline void calcLCM ( const mpz_class& a, const mpz_class& b ) {

            register_p = a;
            register_q = b;

            mpz_lcm( register_y.get_mpz_t(), register_p.get_mpz_t(), register_q.get_mpz_t() );
		}

		inline void startIncrementalLCM ( const mpz_class& initial ) {
			register_y = initial;
		}

		inline void calcIncrementalLCM ( const mpz_class& next ) {

			if ( mpz_cmp_ui( next.get_mpz_t(), 0 ) != 0 ) {
                mpz_lcm( register_y.get_mpz_t(), register_y.get_mpz_t(), next.get_mpz_t() );
			}
		}
		
		inline void calcGCD ( const mpz_class& a, const mpz_class& b ) {
            mpz_gcd( register_y.get_mpz_t(), a.get_mpz_t(), b.get_mpz_t() );
		}

		inline void calcGCD ( mpz_class & res, const mpz_class& a, const mpz_class& b ) {
            mpz_gcd( res.get_mpz_t(), a.get_mpz_t(), b.get_mpz_t() );
		}

		inline void calcExendedGCD ( const mpz_class& a, const mpz_class& b ) {
            register_p = a;
            register_q = b;

            mpz_gcdext( register_y.get_mpz_t(), register_s.get_mpz_t(), register_t.get_mpz_t(), register_p.get_mpz_t(), register_q.get_mpz_t() );            
        }

		inline void startIncrementalGCD ( const mpz_class& initial ) {
			register_y = initial;
		}
		
		inline void calcIncrementalGCD ( const mpz_class& next ) {
            if ( next != 0 )
                mpz_gcd( register_y.get_mpz_t(), register_y.get_mpz_t(), next.get_mpz_t() );
		}

		inline const mpz_class& getResult () const {
			return register_y;
		}
		
		inline const mpz_class& getRegisterS () const {
			return register_s;
		}
		
		inline const mpz_class& getRegisterT () const {
			return register_t;
		}

		inline const mpz_class& getRegisterP () const {
			return register_p;
		}
		
		inline const mpz_class& getRegisterQ () const {
			return register_q;
		}

		inline const mpz_class& getRegisterG () const {
			return register_g;
		}

		inline void loadRegisterG ( const mpz_class& value ) {
			register_g = value;
		}

		inline void loadRegisterP ( const mpz_class& value ) {
			register_p = value;
		}

		inline void loadRegisterP ( const long value ) {
			register_p = value;
		}

		inline void divideRegisterP ( const mpz_class& divisor ) {
			register_p /= divisor;
		}

		inline void loadRegisterQ ( const mpz_class& value ) {
			register_q = value;
		}

		inline void multiplyRegisterQ ( const mpz_class& scalar ) {
			register_q *= scalar;
		}

		inline void loadRegisterQ ( const long value ) {
			register_q = value;
		}

        inline mpz_class & q () {
            return register_q;
        }

		inline void divideRegisterQ ( const mpz_class& divisor ) {
			register_q /= divisor;
		}

        inline const mpz_class & get_register_s () const {
            return register_s;
        }

        inline mpz_class & get_register_s () {
            return register_s;
        }

        inline mpz_class & get_register_t () {
            return register_t;
        }

        inline const mpz_class & get_register_t () const {
            return register_t;
        }

		inline void loadRegisterS ( const mpz_class& value ) {
			register_s = value;
		}

		inline void loadRegisterS ( const long value ) {
			register_s = value;
		}

		inline void multiplyRegisterS ( const mpz_class& scalar ) {
			register_s *= scalar;
		}

		inline void subtractRegisterS ( const mpz_class& value ) {
			register_s -= value;
		}

		inline void divideRegisterS ( const mpz_class& divisor ) {
			register_s /= divisor;
		}

		inline void negateRegisterS () {
			register_s = -register_s;
		}
		
		inline void incrementRegisterS ( const mpz_class& increment ) {
			register_s += increment;
		}
	
		inline void loadRegisterT ( const mpz_class& value ) {
			register_t = value;
		}

		inline void loadRegisterT ( const long value ) {
			register_t = value;
		}
		
		inline void incrementRegisterT ( const mpz_class& increment ) {
			register_t += increment;
		}

		inline void subtractRegisterT ( const mpz_class& value ) {
			register_t -= value;
		}

		inline void multiplyRegisterT ( const mpz_class& scalar ) {
			register_t *= scalar;
		}

		inline void divideRegisterT ( const mpz_class & divisor ) {
			register_t /= divisor;
		}

		inline void convertResultToAbsoluteValue () {
			register_y = abs( register_y );
		}

		inline void convertResultToNegativeValue () {
			register_y = -abs( register_y );
		}
	};
}
#endif // __DD_GDC_MODULE__
