/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_COMB_TEST_SINGLE_SET_RAY_ENUMERATOR__
#define __DD_COMB_TEST_SINGLE_SET_RAY_ENUMERATOR__

#include "DDExtremeRayEnumerator.h"
#include "DDCombTestTraverseVisitor.h"
#include "DDCombTestSingleSetRayCacheEntry.h"

#define __DD_COMB_TEST_SINGLE_SET_RAY_ENUMERATOR_CONTAINERS__ 6

namespace addibit {
    
    class CombTestSingleSetRayEnumerator : public ExtremeRayEnumerator<SingleSetRay, CombTestSingleSetRayCacheEntry, PositiveCombTestTraverseVisitor, NegativeCombTestTraverseVisitor> {
    
        private:
        AbstractTreeCreator * mLDVerCreator;
        AbstractTreeCreator * mHDVerCreator;
        
        const TreeNode * mHDVerificator;
        const TreeNode * mLDVerificator;
        
        uint_fast16_t __adjacency_stack_size;
    
        inline void init () {
            const Options & options = Options::current();
            
            if ( !mCacheNode.data().verification_ld().empty() ) {
                const bool trivial_container = mCacheNode.data().verification_ld().size() < options.ld_verification().min_elements();
                
                if ( trivial_container ) {
                    mLDVerCreator = new LeafTreeCreator();
                } else {
                    const BitSelectStrategy * ld_strategy = mStrategyFactory.createVerificationStrategy( mCacheNode.data().verification_ld(),
                                                                                                         mCacheNode.data().positive_ld(),
                                                                                                         mCacheNode.data().negative_ld(),
                                                                                                         options.ld_verification().bpt_bit_selection_strategy() );
                
                    mLDVerCreator = mTreeFactory.getTreeCreator( options.ld_verification(), *ld_strategy );
                    delete ld_strategy;
                }
                
                mLDVerCreator->setTarget( VERIFICATION );
                mLDVerificator = mLDVerCreator->createLDTree( mCacheNode.data().verification_ld().export_refs( mCacheNode.data().pattern() ) );

    #ifdef __OUTPUT_TREES__
                mLDVerificator->toFile( "ld_ver_tree", plane(), 6 );
    #endif
    
                __adjacency_stack_size = mLDVerCreator->getAdjacencyStackSize();
            } else {
                mLDVerificator = NULL;
            }
            
            if ( !mCacheNode.data().verification_hd().empty() ) {
                const bool trivial_container = mCacheNode.data().verification_hd().size() < options.hd_verification().min_elements();
                
                if ( trivial_container ) {
                    mHDVerCreator = new LeafTreeCreator();
                } else {
                    const BitSelectStrategy * hd_strategy = mStrategyFactory.createVerificationStrategy( mCacheNode.data().verification_hd(),
                                                                                                         mCacheNode.data().positive_hd(),
                                                                                                         mCacheNode.data().negative_hd(),
                                                                                                         options.hd_verification().bpt_bit_selection_strategy() );
                
                    mHDVerCreator = mTreeFactory.getTreeCreator( options.hd_verification(), *hd_strategy );
                    delete hd_strategy;
                }
                
                mHDVerCreator->setTarget( VERIFICATION );                
                mHDVerificator = mHDVerCreator->createHDTree( mCacheNode.data().verification_hd().export_refs( mCacheNode.data().pattern() ) );
                
    #ifdef __OUTPUT_TREES__
                mHDVerificator->toFile( "hd_ver_tree", plane(), 6 );
    #endif
    
                if ( __adjacency_stack_size < mHDVerCreator->getAdjacencyStackSize() ) {
                    __adjacency_stack_size = mHDVerCreator->getAdjacencyStackSize();
                }
            } else {
                mHDVerificator = NULL;
            }
        }
    
        protected:
        void run_adjacency_test ( AbstractTreeCreator * const, StepStats & );   
        
    #ifdef __STATS_OUTPUT__
        void fill_stats ( struct StepStats & );  
    #endif
      
        inline PositiveCombTestTraverseVisitor positive_traverse_visitor ( const TreeNode * const tree,
                                                                           const RayWrapperContainer<SingleSetRay> & container,
                                                                           const uint_fast16_t stack_size ) const {
            return PositiveCombTestTraverseVisitor( tree, container, std::max( __adjacency_stack_size, stack_size ), mPlane, mHDVerificator, mLDVerificator, mCacheNode );
        }
        
        inline NegativeCombTestTraverseVisitor negative_traverse_visitor ( const TreeNode * const tree,
                                                                           const RayWrapperContainer<SingleSetRay> & container,
                                                                           const uint_fast16_t stack_size ) const {
            return NegativeCombTestTraverseVisitor( tree, container, std::max( __adjacency_stack_size, stack_size ), mPlane, mHDVerificator, mLDVerificator, mCacheNode );
        }      
      
        public:
        CombTestSingleSetRayEnumerator ( const Matrix & system,
                                         const uint_fast16_t plane,
                                         CacheEntryNode<SingleSetRay, CombTestSingleSetRayCacheEntry> * node ) :
            ExtremeRayEnumerator<SingleSetRay, CombTestSingleSetRayCacheEntry, PositiveCombTestTraverseVisitor, NegativeCombTestTraverseVisitor>( system, plane, node, __DD_COMB_TEST_SINGLE_SET_RAY_ENUMERATOR_CONTAINERS__ ),
            mLDVerCreator( NULL ),
            mHDVerCreator( NULL ),
            __adjacency_stack_size( 0 ) {
            
            init();
        }
        
        ~CombTestSingleSetRayEnumerator () {
            if ( mLDVerCreator ) {
                delete mLDVerCreator;
            }
            if ( mHDVerCreator ) {
                delete mHDVerCreator;
            }
        }
    };
}

#endif /* __DD_COMB_TEST_SINGLE_SET_RAY_ENUMERATOR__ */
