/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_COMB_TEST_TRAVERSE_VISITOR__
#define __DD_COMB_TEST_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDSingleSetTraverseVisitor.h"
#include "DDSingleSetRay.h"
#include "DDCombTestSingleSetRayCacheEntry.h"
#include "DDEdgeCandidatesContainer.h"

namespace addibit {

    class CombTestTraverseVisitor : public SingleSetTraverseVisitor<CombTestSingleSetRayCacheEntry> {

        private:
        const TreeNode * const mHighDegVerificator;
        const TreeNode * const mLowDegVerificator;
        
        const uint_fast16_t mPopulationSplitCoeff;

        virtual void create_shuffled ( const SingleSetRay * const, const SingleSetRay * const ) = 0;

        protected:
        using SingleSetTraverseVisitor<CombTestSingleSetRayCacheEntry>::mCandidatesContainer;
        using SingleSetTraverseVisitor<CombTestSingleSetRayCacheEntry>::narrow;

        public:
        CombTestTraverseVisitor( const TreeNode * const tree,
                                 const RayWrapperContainer<SingleSetRay> & container,
                                 const uint_fast16_t stack_size,
                                 const uint_fast16_t plane,
                                 const TreeNode * const hd_verificator,
                                 const TreeNode * const ld_verificator,
                                 CacheEntryNode<SingleSetRay, CombTestSingleSetRayCacheEntry> & node ) :
            SingleSetTraverseVisitor<CombTestSingleSetRayCacheEntry>( tree, container, stack_size, plane, node ),
            mHighDegVerificator( hd_verificator ),
            mLowDegVerificator( ld_verificator ),
            mPopulationSplitCoeff( node.data().verification_deg_threshold() - 1 ) {}    
        
        void verify_edge ( const SingleSetRay * const ray, const AdjacencyCandidate * const candidate ) {
            const SingleSetRay * const r_2 = reinterpret_cast<const SingleSetRay *>( candidate->id() );
            
            if ( are_trivially_adjacent( ray, r_2, candidate ) ) {
                create_shuffled( ray, r_2 );
            } else { 
                EdgeCandidate face( ( uintptr_t ) ray, ( uintptr_t ) r_2, *candidate, mCandidatesContainer.stack() );
    
    #ifdef __STATS_OUTPUT__
                TreeNodeCondition::NONTRIVIAL_CANDIDATES++;
    #endif
                if ( ( mHighDegVerificator == NULL || !mHighDegVerificator->verify( face )) && ( face.population() > mPopulationSplitCoeff || mLowDegVerificator == NULL || !mLowDegVerificator->verify( face )) ) {
                    create_shuffled( ray, r_2 );
                }
            }
        }

        inline void process_ray ( const SingleSetRay & ray ) {
            narrow( ray.getActiveSet() );            
            mCandidatesContainer.verify<SingleSetRay, CombTestTraverseVisitor>( &ray, *this );
        } 
    };
    
    class PositiveCombTestTraverseVisitor : public CombTestTraverseVisitor {
        
        protected:
        using SingleSetTraverseVisitor::create;
        
        void create_shuffled ( const SingleSetRay * const negative, const SingleSetRay * const positive ) {
            create( positive, negative );            
        }
        
        public:
        PositiveCombTestTraverseVisitor( const TreeNode * const tree,
                                         const RayWrapperContainer<SingleSetRay> & container,
                                         const uint_fast16_t stack_size,
                                         const uint_fast16_t plane,
                                         const TreeNode * const hd_verificator,
                                         const TreeNode * const ld_verificator,
                                         CacheEntryNode<SingleSetRay, CombTestSingleSetRayCacheEntry> & node ) :
            CombTestTraverseVisitor( tree, container, stack_size, plane, hd_verificator, ld_verificator, node ) {}

    };

    class NegativeCombTestTraverseVisitor : public CombTestTraverseVisitor {
        
        protected:
        using SingleSetTraverseVisitor::create;
                
        void create_shuffled ( const SingleSetRay * const positive, const SingleSetRay * const negative ) {            
            create( positive, negative );            
        }
        
        public:
        NegativeCombTestTraverseVisitor( const TreeNode * const tree,
                                         const RayWrapperContainer<SingleSetRay> & container,
                                         const uint_fast16_t stack_size,
                                         const uint_fast16_t plane,
                                         const TreeNode * const hd_verificator,
                                         const TreeNode * const ld_verificator,
                                         CacheEntryNode<SingleSetRay, CombTestSingleSetRayCacheEntry> & node ) :
            CombTestTraverseVisitor( tree, container, stack_size, plane, hd_verificator, ld_verificator, node ) {}
    };
}

#endif // __DD_COMB_TEST_TRAVERSE_VISITOR__
