/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_CONDITIONED_RAY_REFERENCE_CONTAINER_LIST__
#define __DD_CONDITIONED_RAY_REFERENCE_CONTAINER_LIST__

#include "DDAbstractConditionedRayReferenceContainer.h"
#include "DDConditionedRayReferenceContainerListVisitor.h"

namespace addibit {

    class ConditionedRayReferenceContainerList {
    
        private:
        std::vector<AbstractConditionedRayReferenceContainer const * > mElements;
        
        public:
        ConditionedRayReferenceContainerList () {}
        
        ConditionedRayReferenceContainerList ( const ConditionedRayReferenceContainerList & other ) :
            mElements( other.mElements ) {}
        
        ~ConditionedRayReferenceContainerList () {
            for ( std::vector<AbstractConditionedRayReferenceContainer const * >::const_iterator it_container = mElements.begin(); it_container != mElements.end(); it_container++ ) {
                delete *it_container;
            }
        }
        
        void insert ( AbstractConditionedRayReferenceContainer const * const container ) {
            mElements.push_back( container );
        }
        
        AbstractConditionedRayReferenceContainer const & operator[] ( const unsigned int index ) const {
            return *( mElements[ index ] );
        }
        
        void iterate ( ConditionedRayReferenceContainerListVisitor & visitor ) const {
            for ( std::vector<AbstractConditionedRayReferenceContainer const * >::const_iterator it_container = mElements.begin(); it_container != mElements.end(); it_container++ ) {
                visitor.onVisit( **it_container );
            }            
        }
        
        const std::string toString () const;
    };
}

#endif /* __DD_CONDITIONED_RAY_REFERENCE_CONTAINER_LIST__ */
