/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_CONTAINER_BINARY_FRAGMENTATION_MAPPING__
#define __DD_CONTAINER_BINARY_FRAGMENTATION_MAPPING__

#include <vector>

#include "DDBitList.h"

namespace addibit {

    class ContainerBinaryFragmentationMapping {
        
        private:
        
        std::vector<BitList> mLowerBound;
        std::vector<BitList> mUpperBound;
        
        unsigned long getReductionMap ( const unsigned long map,
                                        const unsigned long ub ) const {
#if defined _LP64 || defined __LP64__
            return 0xAAAAAAAAAAAAAAAA & map & ( map << 1 ) & ~ub;  
#else
			return 0xAAAAAAAA & map & ( map << 1 ) & ~ub;                         
#endif // defined _LP64 || defined __LP64__
        }
        
        public:
        ContainerBinaryFragmentationMapping ( const unsigned int blocks )  {}
            
         void insert_empty () {
            mLowerBound.push_back( BitList( __BIT_LIST_ALLOC_SPACE__, ULONG_MAX ) );
            mUpperBound.push_back( BitList( __BIT_LIST_ALLOC_SPACE__, 0 ) );
        }    
            
        void update ( const unsigned int id,
                      const BitList & map ) {
            mLowerBound[ id ] &= map;
            mUpperBound[ id ] |= map;
        }
        
        const BitList & lbound ( const unsigned int id ) const {
            return mLowerBound[ id ];
        }
        
        const BitList & ubound ( const unsigned int id ) const {
            return mUpperBound[ id ];
        }
        
        const BitList getReductionMap ( const unsigned int id,
                                        const BitList & map ) const {
            BitList reduction( __BIT_LIST_ALLOC_SPACE__ );
            const BitList & ub = ubound( id );
            
            reduction[ 0 ] = getReductionMap( map[ 0 ], ub[ 0 ] );
            
            for ( uint_fast16_t i = 1; i < __DD_BLOCK_COUNT__; i++ ) {
                reduction[ i ] = getReductionMap( map[ i ], ub[ i ] );
            }
            
            return reduction;
        }
        
        const BitList findOpenFragment ( const unsigned int id ) const {
            const BitList range = mUpperBound[ id ] ^ mLowerBound[ id ];
            BitList fragment( __BIT_LIST_ALLOC_SPACE__ );
                        
            for ( fragment[ 0 ] = 0x0000000000000003; !range.contains( fragment ); fragment <<= 2 );
            
            return fragment;
        }
    };
}

#endif /* __DD_CONTAINER_BINARY_FRAGMENTATION_MAPPING__ */
