/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_CONTAINER_BIT_PATTERN_MAPPING__
#define __DD_CONTAINER_BIT_PATTERN_MAPPING__

#include <vector>

#include "DDBitList.h"
#include "DDRayRefVisitor.h"

namespace addibit {

    class ContainerBitPatternMapping : public RayRefVisitor {
    
        private:
        std::vector<BitList> mUnionMap;
        std::vector<BitList> mCutMap;
        
        public:
        
        void insert_empty () {
            mUnionMap.push_back( BitList( __BIT_LIST_ALLOC_SPACE__, 0 ) );
            mCutMap.push_back( BitList( __BIT_LIST_ALLOC_SPACE__, ULONG_MAX ) );
        }        
        
        void update ( const unsigned int id, const BitList & vector ) { 
            mCutMap[ id ] &= vector;
            mUnionMap[ id ] |= vector;
        }            
        
        const BitList & cut_map ( const uint_fast32_t id ) const {
            return mCutMap[ id ];
        }
        
        const BitList & union_map ( const uint_fast32_t id ) const {
            return mUnionMap[ id ];
        }
        
        void onVisit ( const RayReference & ref ) {
            update( 0, ref.activeSet );
        }
        
        void reset () {
            mUnionMap.clear();
            mCutMap.clear();
        }
    };
}

#endif /* __DD_CONTAINER_BIT_PATTERN_MAPPING__ */
