/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DDDifferentialBitStrategy.h"

using namespace addibit;

std::vector<unsigned int> DifferentialBitStrategy::createMapping () const {

	std::vector<unsigned int> mapping;
	mapping.reserve( mPlane );

	const double_t * const & list_probability = probability();

	for ( uint_fast16_t index = 0; index < mPlane; index++ ) {
		uint_fast16_t position = 0;

		for ( uint_fast16_t prev = 0; prev < index; prev++ ) {
            const double_t prev_coeff = getProbabilityCoeff( prev, list_probability[ prev ], __tree_probability[ prev ] );
            const double_t index_coeff = getProbabilityCoeff( index, list_probability[ index ], __tree_probability[ index ] );
            
			if ( prev_coeff > index_coeff ) {
				position++;
			}
		}

		mapping.insert( mapping.begin() + position, index );
	}
    
  //  DD_PRINT_VECTOR_ELEMENTARY( mapping )

	return mapping;
}

const std::string DifferentialBitStrategy::toString () const {
    std::stringstream result;
    const double_t * const & list_probability = probability();
    
    for ( uint_fast16_t pos = 0; pos < mPlane; pos++ ) {
        result << std::fixed << "[" << list_probability[ pos ] << ", " << __tree_probability[ pos ] << "] ";
    }
    result << std::endl;
    
    return result.str();
}
