
#ifndef __DD_DUAL_SET_FACE__
#define __DD_DUAL_SET_FACE__

#include "DDDualSetRay.h"

#include "DDLogging.h"

namespace addibit {

	class DualSetFace {

		private:
		const DualSetRay* mFirst;
		const DualSetRay* mSecond;
		BitList mPrimaryActiveSet;
		BitList mSecondaryActiveSet;

		public:
		DualSetFace( const DualSetRay * const first,
                     const DualSetRay * const second ) :
			mFirst( first ),
			mSecond( second ),
			mPrimaryActiveSet( __BIT_LIST_ALLOC_SPACE__ ),
			mSecondaryActiveSet( __BIT_LIST_ALLOC_SPACE__ ) {}
/*
		DualSetFace( const DualSetRay* const first, const DualSetRay* const second, const BitList & primaryActiveSet, const BitList & secondaryActiveSet ) :
			mFirst( first ),
			mSecond( second ),
			mPrimaryActiveSet( primaryActiveSet ),
			mSecondaryActiveSet( secondaryActiveSet ) {}*/

		const DualSetRay* getFirst () const {
			return mFirst;
		}

		const DualSetRay* getSecond () const {
			return mSecond;
		}

		const BitList& getPrimaryActiveSet () const {
			return mPrimaryActiveSet;
		}

		void setPrimaryActiveSet ( const BitList & map, const uint_fast16_t plane ) {
			mPrimaryActiveSet = map;
			mPrimaryActiveSet.setBit( plane );
		}

		const BitList& getSecondaryActiveSet () const {
			return mSecondaryActiveSet;
		}

		void setSecondaryActiveSet ( const BitList & map ) {
			mSecondaryActiveSet = map;
		}

		const BitList getActiveSet () const {
			return mFirst->getActiveSet() & mSecond->getActiveSet();
		}

		LinearDependenceEntry** getInitialInternalMap ( const uint_fast16_t size, LinearDependenceCache & cache ) const {

			LinearDependenceEntry** internal_map = new LinearDependenceEntry*[ size - GLOBAL_RAY_DIM ] - GLOBAL_RAY_DIM;

			for ( uint_fast16_t block = 0; block < __DD_BLOCK_COUNT__; block++ ) {
				__BIT_LIST_RAW_DATA_TYPE__ block_data = mSecondaryActiveSet[ block ];

				while ( block_data ) {
					const uint_fast16_t index = block * __BLOCK_SIZE__ + __builtin_ctzl( block_data );

					if ( mPrimaryActiveSet.contains ( mFirst->getLinearDependenceEntry( index )->getMap() ) ) {
						internal_map[ index ] = mFirst->link( index );
					} else if ( mPrimaryActiveSet.contains ( mSecond->getLinearDependenceEntry( index )->getMap() ) ) {
						internal_map[ index ] = mSecond->link( index );
					} else {
						internal_map[ index ] = cache.acquireLinearDependenceEntry( mPrimaryActiveSet, index );
					}

                    __REMOVE_LS_BIT( block_data );
				}
			}

			return internal_map;
		}

		const mpz_class & getPlaneCoeff ( const uint_fast16_t plane ) const {

				UtilsFactory::CALC_UTILS.loadRegisterS( UtilsFactory::CALC_UTILS.getRegisterP() );
				UtilsFactory::CALC_UTILS.multiplyRegisterS( mSecond->getCachedPlaneCoeff( plane ) );

				UtilsFactory::CALC_UTILS.loadRegisterT( UtilsFactory::CALC_UTILS.getRegisterQ() );
				UtilsFactory::CALC_UTILS.multiplyRegisterT( mFirst->getCachedPlaneCoeff( plane ) );	

				UtilsFactory::CALC_UTILS.subtractRegisterS( UtilsFactory::CALC_UTILS.getRegisterT() );	

				return UtilsFactory::CALC_UTILS.getRegisterS();
		}

		const DualSetRay * getIntersection ( const uint_fast16_t plane,
                                             const uint_fast16_t total_planes,
                                             LinearDependenceCache & cache ) const {
                                                 
			LinearDependenceEntry** internal_map = getInitialInternalMap( total_planes, cache );
			DualSetRay * ray = new DualSetRay( mPrimaryActiveSet, mSecondaryActiveSet, internal_map, plane, total_planes );

			UtilsFactory::CALC_UTILS.calcGCD( mFirst->getCachedPlaneCoeff( plane ), mSecond->getCachedPlaneCoeff( plane ) );

			UtilsFactory::CALC_UTILS.loadRegisterP( mFirst->getCachedPlaneCoeff( plane ) );
			UtilsFactory::CALC_UTILS.divideRegisterP( UtilsFactory::CALC_UTILS.getResult() );

			UtilsFactory::CALC_UTILS.loadRegisterQ( mSecond->getCachedPlaneCoeff( plane ) );
			UtilsFactory::CALC_UTILS.divideRegisterQ( UtilsFactory::CALC_UTILS.getResult() );

/*
#ifndef __COMPRESSION__
			for ( unsigned int index = 0; index < ray->getDimension(); index++ ) {
				if ( mFirst->isNonZeroAt( index ) ) {

					UtilsFactory::CALC_UTILS.loadRegisterS( mFirst->getValue( index ) );

					if ( mSecond->isNonZeroAt( index ) ) {
						UtilsFactory::CALC_UTILS.loadRegisterT( mSecond->getValue( index ) ); 
						UtilsFactory::CALC_UTILS.multiplyRegisterT( UtilsFactory::CALC_UTILS.getRegisterP() );
	
						UtilsFactory::CALC_UTILS.multiplyRegisterS( UtilsFactory::CALC_UTILS.getRegisterQ() );
						UtilsFactory::CALC_UTILS.subtractRegisterT( UtilsFactory::CALC_UTILS.getRegisterS() );
						ray->setValue( index, UtilsFactory::CALC_UTILS.getRegisterT() );
					} else {
						UtilsFactory::CALC_UTILS.multiplyRegisterS( UtilsFactory::CALC_UTILS.getRegisterQ() );
						UtilsFactory::CALC_UTILS.negateRegisterS();
						ray->setValue( index, UtilsFactory::CALC_UTILS.getRegisterS() );
					}
			
				} else if ( mSecond->isNonZeroAt( index ) ) {
					UtilsFactory::CALC_UTILS.loadRegisterT( mSecond->getValue( index ) );
					UtilsFactory::CALC_UTILS.multiplyRegisterT( UtilsFactory::CALC_UTILS.getRegisterP() );
					ray->setValue( index, UtilsFactory::CALC_UTILS.getRegisterT() );
				} 
			} 
#endif // __COMPRESSION__
*/
			unsigned int next_plane;

			for ( next_plane = plane + 1; likely( next_plane < total_planes ); next_plane++ ) {
				const mpz_class & coeff = getPlaneCoeff( next_plane );
				ray->setCachedPlaneCoeff( next_plane, coeff );
	
				if ( coeff == 0 ) {
					ray->setLinearDependenceEntry( next_plane, cache.acquireLinearDependenceEntry( mPrimaryActiveSet, next_plane ) );
				} else if ( coeff < 0 ) {
					break;
				}
			}

			for ( ; likely( next_plane < total_planes ); next_plane++ ) {
				ray->setCachedPlaneCoeff( next_plane, getPlaneCoeff( next_plane ) );
			} 

			cache.update();

			return ray;
		}

	};
}

#endif // __DD_DUAL_SET_FACE__
