/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DDDualSetRayCacheEntry.h"

using namespace addibit;

DualSetRayCacheEntry * DualSetRayCacheEntry::inequality_entry ( const uint_fast16_t rows,
                                                                const uint_fast16_t plane,
                                                                const BitList & pattern ) {
    return new InequalityDualSetRayCacheEntry( rows, plane, pattern );                          
}

DualSetRayCacheEntry * DualSetRayCacheEntry::inequality_entry ( const uint_fast16_t rows,
                                                                const uint_fast16_t plane,
                                                                const uint_fast16_t plane_coeff_offset,
                                                                const BitList & pattern ) {
    return new InequalityDualSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern );                          
}

DualSetRayCacheEntry * DualSetRayCacheEntry::equality_entry ( const uint_fast16_t rows,
                                                              const uint_fast16_t plane,
                                                              const uint_fast16_t plane_coeff_offset,
                                                              const BitList & pattern ) {
    return new EqualityDualSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern );                          
}

bool InequalityDualSetRayCacheEntry::proceed ( std::shared_ptr<const DualSetRay> ray ) {
    return ray->isOnThePlane( plane() ) || CacheEntry<DualSetRay>::insert_narrow( ray );
}

bool EqualityDualSetRayCacheEntry::proceed ( std::shared_ptr<const DualSetRay> ray ) {
    
    if ( !ray->isOnThePlane( plane() ) ) {
        CacheEntry<DualSetRay>::insert_narrow( ray );
        return false;
    } else {
        return true;
    }
}
