/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_DUAL_SET_RAY_CACHE_ENTRY__
#define __DD_DUAL_SET_RAY_CACHE_ENTRY__

#include "DDDualSetRay.h"
#include "DDCacheEntry.h"

namespace addibit {

	class DualSetRayCacheEntry : public CacheEntry<DualSetRay> {

		public:
		DualSetRayCacheEntry ( const uint_fast16_t rows,
                               const uint_fast16_t plane,
                               const BitList & pattern ) :
			CacheEntry<DualSetRay>( rows, plane, pattern ) {}
            
		DualSetRayCacheEntry ( const uint_fast16_t rows,
                               const uint_fast16_t plane,
                               const uint_fast16_t plane_coeff_offset,
                               const BitList & pattern ) :
			CacheEntry<DualSetRay>( rows, plane, plane_coeff_offset, pattern ) {}            

		virtual ~DualSetRayCacheEntry () {}
		
		static DualSetRayCacheEntry * inequality_entry ( const uint_fast16_t, const uint_fast16_t, const BitList & );
        static DualSetRayCacheEntry * inequality_entry ( const uint_fast16_t, const uint_fast16_t, const uint_fast16_t, const BitList & );
		static DualSetRayCacheEntry * equality_entry ( const uint_fast16_t, const uint_fast16_t, const uint_fast16_t, const BitList & );
	};

	class InequalityDualSetRayCacheEntry : public DualSetRayCacheEntry {
	
	    public:
		InequalityDualSetRayCacheEntry ( const uint_fast16_t rows,
                                         const uint_fast16_t plane,
                                         const BitList & pattern ) :
			DualSetRayCacheEntry( rows, plane, pattern ) {}	

		InequalityDualSetRayCacheEntry ( const uint_fast16_t rows,
                                         const uint_fast16_t plane,
                                         const uint_fast16_t plane_coeff_offset,
                                         const BitList & pattern ) :
			DualSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern ) {}				
            
		~InequalityDualSetRayCacheEntry () {}	    
			
		bool proceed ( std::shared_ptr<const DualSetRay> );
	};
	
	class EqualityDualSetRayCacheEntry : public DualSetRayCacheEntry {
	
	    public:
		EqualityDualSetRayCacheEntry ( const uint_fast16_t rows,
                                       const uint_fast16_t plane,
                                       const uint_fast16_t plane_coeff_offset,
                                       const BitList & pattern ) :
			DualSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern ) {}	  
			
		~EqualityDualSetRayCacheEntry () {}
			
		bool proceed ( std::shared_ptr<const DualSetRay> );	  
	};
}

#endif /* __DD_DUAL_SET_RAY_CACHE_ENTRY__ */
