/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_DUAL_SET_RAY_ENUMERATOR__
#define __DD_DUAL_SET_RAY_ENUMERATOR__

#include "DDExtremeRayEnumerator.h"
#include "DDPartTestTraverseVisitor.h"

#define __DD_DUAL_SET_RAY_ENUMERATOR_CONTAINERS__ 4

namespace addibit {

    class DualSetRayEnumerator : public ExtremeRayEnumerator<DualSetRay, DualSetRayCacheEntry, PositivePartTestTraverseVisitor, NegativePartTestTraverseVisitor> {

        private:
        LinearDependenceCache & mDependenceCache;
        FaceProver & mProver;

        protected:

        inline PositivePartTestTraverseVisitor positive_traverse_visitor ( const TreeNode * const tree,
                                                                           const RayWrapperContainer<DualSetRay> & container,
                                                                           const uint_fast16_t stack_size ) const {
            return PositivePartTestTraverseVisitor( tree, container, stack_size, mPlane, mCacheNode, mDependenceCache, mProver );
        }
        
        inline NegativePartTestTraverseVisitor negative_traverse_visitor ( const TreeNode * const tree,
                                                                           const RayWrapperContainer<DualSetRay> & container,
                                                                           const uint_fast16_t stack_size ) const {
            return NegativePartTestTraverseVisitor( tree, container, stack_size, mPlane, mCacheNode, mDependenceCache, mProver );
        }

        public:
        DualSetRayEnumerator ( const Matrix & system,
                               const uint_fast16_t plane,
                               CacheEntryNode<DualSetRay, DualSetRayCacheEntry> * node,
                               LinearDependenceCache & dependencies,
                               FaceProver & prover ) :
            ExtremeRayEnumerator<DualSetRay, DualSetRayCacheEntry,PositivePartTestTraverseVisitor, NegativePartTestTraverseVisitor>( system, plane, node, __DD_DUAL_SET_RAY_ENUMERATOR_CONTAINERS__ ),
            mDependenceCache( dependencies ),
            mProver( prover ) {}
    };
}

#endif // __DD_DUAL_SET_RAY_ENUMERATOR__
