/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_DUAL_SET_RAY_GENERATOR__
#define __DD_DUAL_SET_RAY_GENERATOR__

#include <vector>

#include "DDExtremeRayGenerator.h"

#include "DDDualSetRay.h"
#include "DDDualSetRayCacheEntry.h"
#include "DDRepresentationVector.h"

#include "DDLinearDependenceCache.h"
#include "DDFaceProver.h"

namespace addibit {

	class DualSetRayGenerator : public ExtremeRayGenerator<DualSetRay, DualSetRayCacheEntry> {

		private:
		LinearDependenceCache mDependenceCache;
        FaceProver mProver;

		RepresentationVector getInitialVector ( const Matrix & base, const unsigned long positivePlane, BitList & primaryActiveSet ) const;
		mpz_class* getPlaneCoeffs ( const RepresentationVector & vector, const BitList & primaryActiveSet, BitList & secondaryActiveSet, LinearDependenceEntry** & internalMap );
		const LinearDependenceEntry* getLinearDependenceEntry ( const BitList & primarySet, const unsigned long & secondaryBit );

        protected:
		const struct StepStats enumerate ( const unsigned long plane, CacheEntryNode<DualSetRay, DualSetRayCacheEntry> * );
        void calculateBasis ( CacheEntryNode<DualSetRay, DualSetRayCacheEntry> * const );
        		
        void fork ( CacheEntryNode<DualSetRay, DualSetRayCacheEntry> * const node ) {
            t_fork<DualSetRayGenerator>( node );
        }                
                
		public:
		DualSetRayGenerator ( const Matrix& system ) :
            ExtremeRayGenerator<DualSetRay, DualSetRayCacheEntry>( system ),
            mDependenceCache ( system ),
            mProver( system.getRows(), system.getColumns() - 2 ) {}
            
		DualSetRayGenerator ( const Matrix & system,
		                      CacheEntryNode<DualSetRay, DualSetRayCacheEntry> * const node ) :
            ExtremeRayGenerator<DualSetRay, DualSetRayCacheEntry>( system, node ),
            mDependenceCache ( system ),
            mProver( system.getRows(), system.getColumns() - 2 ) {}

		~DualSetRayGenerator () {}
	};
}

#endif // __DD_DUAL_SET_RAY_GENERATOR_
