/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDExtremeRayUncompressor.h"

using namespace addibit;

void ExtremeRayUncompressor::calcExtremeRay ( const Matrix & matrix,
                                              const BitList & map,
                                              OutputVector<double> & result ) {
    
    const uint_fast16_t a_data_cols = matrix.getColumns();
    const uint_fast16_t a_data_rows = map.getPopulationCount() + 1;

    double * a_data = new double[ a_data_cols * a_data_rows ];
    
    load( matrix, map, a_data );

    a_data[ a_data_cols * ( a_data_rows - 1 ) ] = 1;
    for ( uint_fast16_t i = 1; i < matrix.getColumns(); i++ ) {
        a_data[ a_data_cols * ( a_data_rows - 1 ) + i ] = 0;
    }

    double * b_data = new double[ a_data_rows ];

    for ( uint_fast16_t i = 0; i < a_data_rows - 1; i++ ) {
        b_data[ i ] = 0;
    }            

    b_data[ a_data_rows - 1 ] = 1;

    gsl_matrix_view m = gsl_matrix_view_array ( a_data, a_data_rows, a_data_cols );
    gsl_vector_view b = gsl_vector_view_array ( b_data, a_data_rows );
    gsl_vector * x = gsl_vector_alloc ( a_data_cols );
    gsl_vector * residual = gsl_vector_alloc ( a_data_rows );

    gsl_vector * tau = gsl_vector_alloc ( a_data_cols );
    gsl_linalg_QR_decomp ( &m.matrix, tau );
    gsl_linalg_QR_lssolve ( &m.matrix, tau, &b.vector, x, residual );

    for ( uint_fast16_t i = 0; i < a_data_cols; i++ ) {
        result.setValue( i, gsl_vector_get( x, i ) );
    }

    gsl_vector_free (tau);
    gsl_vector_free (x);
    gsl_vector_free (residual);
    delete[] a_data;
    delete[] b_data;
}
