/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_FRAGMENTATION_DATA__
#define __DD_FRAGMENTATION_DATA__

#include <stdint.h>
#include <new>
#include <memory>

#include "DDTweaks.h"

namespace addibit {

    union FragmentationSegment {
        __m128i __data;
        uint8_t __vec[16];
    };

    class FragmentationData {
        
        private:
        uint_fast16_t __segment_count;
        FragmentationSegment * __segment_data;
        
        public:
        FragmentationData () :
            __segment_data( NULL ) {}
        
        FragmentationData ( const uint_fast16_t frag_count ) :
            __segment_count( ( frag_count + 15 ) / 16 ),
            __segment_data( new ( aligned_alloc( 16, 16 * __segment_count ) ) FragmentationSegment[ __segment_count ] ) {
                
            uint_fast16_t i = 0;
            
            do {
                __segment_data[i].__data = _mm_setzero_si128();
            } while( ++i < __segment_count );            
        }

        FragmentationData ( const uint_fast16_t frag_count,
                            const char value ) :
            __segment_count( ( frag_count + 15 ) / 16 ),
            __segment_data( new ( aligned_alloc( 16, 16 * __segment_count ) ) FragmentationSegment[ __segment_count ] ) {
                
            uint_fast16_t i = 0;
            
            do {
                __segment_data[i].__data = _mm_set1_epi8( value );
            } while( ++i < __segment_count );            
        }

        FragmentationData ( const FragmentationData & lvalue ) :
            __segment_count( lvalue.__segment_count ),
            __segment_data( new ( aligned_alloc( 16, 16 * __segment_count ) ) FragmentationSegment[ __segment_count ] ) {
        
            uint_fast16_t i = 0;
            
            do {
                __segment_data[i].__data = lvalue.__segment_data[i].__data;
            } while( ++i < __segment_count ); 
        }
            
        FragmentationData ( FragmentationData && rvalue ) :
            __segment_count( rvalue.__segment_count ),
            __segment_data( rvalue.__segment_data ) {
        
            rvalue.__segment_data = NULL;
        }
        
        ~FragmentationData () {
            if ( __segment_data ) {
                free( __segment_data );
            }
        }
        
        void operator= ( FragmentationData && rvalue ) {
            __segment_count = rvalue.__segment_count;
            std::swap( __segment_data, rvalue.__segment_data );
        }
        
        inline void set_frag_value ( uint8_t value, uint_fast16_t pos ) {            
            __segment_data[0].__vec[ pos ] = value;
        }
        
        inline uint_fast8_t get_frag_value ( const uint_fast16_t pos ) const {
            return __segment_data[0].__vec[ pos ];
        }
        
        inline uint_fast16_t segment_count () const {
            return __segment_count;
        }
        
        void operator &= ( const FragmentationData & lvalue ) {
            uint_fast16_t i = 0;
            
            do {
                __segment_data[i].__data = _mm_min_epu8( __segment_data[i].__data, lvalue[i].__data );
            } while( ++i < __segment_count );
        }
        
        void operator |= ( const FragmentationData & lvalue ) {
            uint_fast16_t i = 0;
            
            do {
                __segment_data[i].__data = _mm_max_epu8( __segment_data[i].__data, lvalue[i].__data );
            } while( ++i < __segment_count );
        }
        
        inline const FragmentationSegment & operator[] ( uint_fast16_t segment ) const {
            return __segment_data[ segment ];
        }
        
        const std::string toString () const;
    }; 
}

#endif /* __DD_FRAGMENTATION_DATA__ */
