/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_FRAGMENTATION_SPLIT_VISITOR__
#define __DD_FRAGMENTATION_SPLIT_VISITOR__

#include "DDAbstractFragmentationSplitVisitor.h"
#include "DDContainerFragmentationMapping.h"
#include "DDTwoBitsFragmentationCondition.h"

#include "DDFragmentationTransformation.h"

namespace addibit {

    class FragmentationSplitVisitor : virtual public AbstractFragmentationSplitVisitor {
    
        private:
        uint_fast16_t mSplittingFragment;
        uint_fast16_t mMaxPopulation;
    
        protected:
        const FragmentationTransformation & mTransformation;
        
        ContainerFragmentationMapping mFragmentationMapping;
        
        void registerContainer ( const unsigned int ) {
            mFragmentationMapping.insert_empty();
        }
        
        unsigned int getTargetContainer ( const RayReference & ref ) {
            const unsigned int c_index = mFragmentationMapping.ref_data( ref.id ).get_frag_value( mSplittingFragment ) <= mMaxPopulation ? 0 : 1;
            mFragmentationMapping.update( set( c_index ).id(), ref );
            return c_index;
        }        
        
        public:                        
        FragmentationSplitVisitor ( const uint_fast16_t depth,
                                    const uint_fast16_t capacity,
                                    const FragmentationTransformation & trans,
                                    const uint_fast16_t fsize ) :
            BitSplitVisitor( depth, capacity, 2 ),
            AbstractFragmentationSplitVisitor( depth, capacity, fsize + 1 ),
            mSplittingFragment( 0 ),
            mTransformation( trans ),
            mFragmentationMapping( mTransformation ){}    
            
        ~FragmentationSplitVisitor () {}            

        void init ( const RayReferenceContainer & container ) {
            AbstractFragmentationSplitVisitor::init( container );
            mFragmentationMapping.init( container.size() );
            container.iterate( mFragmentationMapping );    
        }
        
        void before ( const RayReferenceContainer & container ) {
            AbstractFragmentationSplitVisitor::before( container );
            mSplittingFragment = mFragmentationMapping.getNextOpenFragment( container.id() );
            mMaxPopulation = mFragmentationMapping.lbound( container.id() ).get_frag_value( mSplittingFragment ); /* + mTransformation.probabilities()[ mSplittingFragment ] */ // mFragmentationMapping.median( container.id(), mSplittingFragment ); 
        }            
        
        bool can_proceed ( const unsigned int index ) const { 
            return mFragmentationMapping.getNextOpenFragment( set( index ).id() ) < mTransformation.getFragmentCount();
        }          
                     
        void refineCondition ( const uint_fast32_t index,
                               FragmentationCondition & cond ) const {
                               
            const uint_fast32_t container_id = set( index ).id();              
            cond = FragmentationCondition( mFragmentationMapping.ubound( container_id ) );                                                                    
        }                                        
    };
}

#endif /* __DD_FRAGMENTATION_SPLIT_VISITOR__ */
