/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_INTERMEDIATE_BIT_SET_NODE__
#define __DD_INTERMEDIATE_BIT_SET_NODE__

#include "DDAbstractIntermediateTreeNode.h"
#include "DDLeafNode.h"
#include "DDTreeNodeCondition.h"

#include "DDRayReferenceContainer.h"

namespace addibit {

    template<class CONDITION_TYPE>
	class IntermediateTreeNode : public AbstractIntermediateTreeNode {

		private:  
        const CONDITION_TYPE mCondition;    

		public:
		IntermediateTreeNode ( CONDITION_TYPE && cond,
                               const unsigned int children ) :
	        AbstractIntermediateTreeNode( children ),
            mCondition( std::move( cond ) ) {}

		void narrow ( const NarrowingCandidate & current,
					  EdgeCandidatesContainer & candidates ) const { 

            if ( mCondition.proceed_narrowing( current ) ) {
                AbstractIntermediateTreeNode::narrow( current, candidates );
            }
        }

		bool verify ( const EdgeCandidate & candidate ) const {
            return mCondition.proceed_verification( candidate ) &&
                   AbstractIntermediateTreeNode::verify( candidate );
        }

        const std::string toString () const {
            std::stringstream result;
            
            result << AbstractIntermediateTreeNode::toString() << ", " << mCondition.toString();
            
            return result.str();
        }
	};
}

#endif // __DD_INTERMEDIATE_BIT_SET_NODE__
