/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_LEAF_NODE__
#define __DD_LEAF_NODE__

#include "DDAbstractLeafNode.h"

namespace addibit {

    template<class CONDITION_TYPE>
	class LeafNode : public AbstractLeafNode {

		private:
        const CONDITION_TYPE mCondition;

		public:
		LeafNode ( const RayReferenceContainer & candidates,
                   CONDITION_TYPE && cond ) :
            AbstractLeafNode( candidates ),
            mCondition( std::move( cond ) ) {}

        ~LeafNode () {}

		void narrow ( const NarrowingCandidate & current,
					  EdgeCandidatesContainer & candidates ) const {
                          
            if ( mCondition.proceed_narrowing( current ) ) {
                AbstractLeafNode::narrow( current, candidates );
            }
        }

		bool verify ( const EdgeCandidate & candidate ) const {
            return mCondition.proceed_verification( candidate ) &&
                   AbstractLeafNode::verify( candidate );
        }

        const std::string toString () const {
            std::stringstream result;      
            result << AbstractLeafNode::toString() << ", " << mCondition.toString();
            return result.str();
        }
	};
}

#endif // __DD_LEAF_NODE__
