/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_LEAF_TREE_CREATOR__
#define __DD_LEAF_TREE_CREATOR__

#include <list>

#include "DDAbstractLeafNode.h"
#include "DDAbstractTreeCreator.h"
#include "DDUnconditionedRayReferenceContainer.h"

namespace addibit {

    class LeafTreeCreator : public AbstractTreeCreator {
    
        private:
        std::list<const TreeNode *> __created;
    
        public:
        ~LeafTreeCreator () {
            for ( auto tree : __created ) {
                delete tree;
            }
        }
        
        const TreeNode * createHDTree ( RayReferenceContainer && candidates ) {
            const TreeNode * result = new AbstractLeafNode( candidates );
            __created.push_front( result );
            return result;
        }
        
        const TreeNode * createLDTree ( RayReferenceContainer && candidates ) {
            return createHDTree( std::move( candidates ) );
        }        
        
        const ConditionedRayReferenceContainerList * createList ( RayReferenceContainer && container ) {
            ConditionedRayReferenceContainerList * list = new ConditionedRayReferenceContainerList();
            list->insert( new UnconditionedRayReferenceContainer( std::move( container ) ) );
            return list;
        }
    };
}

#endif /* __DD_LEAF_TREE_CREATOR__ */
