/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_OUTPUT__
#define __DD_OUTPUT__

#include <string>
#include <vector>
#include <fstream>

namespace addibit {

	class Output {

        private:
        std::ofstream mOutput;
		
		public:
        Output ( const char* file ) :
            mOutput( file ) {}

        ~Output () {
            mOutput.close();
        }

        void write ( const std::string & line ) {
            mOutput << line << std::endl;
        }

        template <class DATA_TYPE>
        void write ( DATA_TYPE & ray ) {
            mOutput << ray.toString() << std::endl;
        }
        
        template <typename T>
        Output & operator<< ( const T & t ) {
            mOutput << t;
            return *this;
        }

        template<class DATA_TYPE>
		static void write ( const char* input, const std::vector<DATA_TYPE>& results ) {
			std::string output_file( input );

			std::ofstream output( (output_file + "out").c_str() ) ;
			
			output << results.size() << " extereme rays:" << std::endl;
			for ( unsigned long index = 0; index < results.size(); index++ ) {
				output << results[index].toString() << std::endl;
			}

			output.close();
		}

	};
}

#endif // __DD_OUTPUT__
