/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_PARTITIONED_PATTERN_CONDITION__
#define __DD_PARTITIONED_PATTERN_CONDITION__

#include "DDTreeNodeCondition.h"

namespace addibit {

	class PartitionedPatternCondition : public TreeNodeCondition {

        private:
        unsigned int mFragmentCount;
        BitList * mFragments;
        unsigned int * mFragmentMax;

        public:

        PartitionedPatternCondition ( const BitList & map,
                                      const unsigned int minCommonBits ) :
            TreeNodeCondition( map, minCommonBits ),
            mFragmentCount( 0 ),
            mFragments( NULL ),
            mFragmentMax( NULL ) {}

        PartitionedPatternCondition ( const BitList & map,
                                      const unsigned int minCommonBits,
                                      const unsigned int fcount,
                                      const BitList * fragments,
                                      const unsigned int * fpeaks ) :
            TreeNodeCondition( map, minCommonBits ),
            mFragmentCount( fcount ) {

            fragment( fcount, fragments, fpeaks );
        }

        ~PartitionedPatternCondition () {
            
            if ( likely( mFragmentMax != NULL ) )
                delete[] mFragmentMax;

            if ( likely( mFragments != NULL ) )
                delete[] mFragments;
        }

        bool proceed_narrowing( const RayReference & ref ) const {

    #ifdef __STATS_OUTPUT__
            TreeNodeCondition::NARROWING_OPS++;
    #endif
            const BitList common_pattern = mMap & ref.activeSet;
            unsigned int common_bits = 0;

            for ( unsigned int f_index = 0; f_index < mFragmentCount; f_index++ ) {
                common_bits += std::min<unsigned int>( common_pattern.getCommonPopCnt( mFragment[ f_index] ), mFragmentMax[ f_index ] );
            }

            return common_bits >= mMinCommonBits;
        }

        void fragment ( const unsigned int fcount,
                        const BitList * fragments,
                        const unsigned int * fpeaks ) {
            
            mFragments = new BitList[ fcount ];
            mFragmentMax = new unsigned int[ fcount ];

            for ( unsigned int f_index = 0; f_index < fcount; f_index++ ) {
                mFragments[ f_index ] = fragments[ f_index ];
                mFragmentMax[ f_index ] = fpeaks[ f_index];
            }            
        }
    };
}

#endif /* __DD_PARTITIONED_PATTERN_CONDITION__ */
