
#include "DDPatternGraph.h"

#include "DDLogging.h"

using namespace addibit;

PatternGraph::PatternGraph ( const unsigned int depth, const unsigned int threshold ) {

	const unsigned int size = 1 << depth;

	mMap = new unsigned int*[size];
	mCount = new unsigned int[size];

	mNodeCount = getNodeCount( depth, threshold );
	mNodes = new unsigned int[ mNodeCount ];

	for ( unsigned int src = 0, nod = 0; src < size; src++ ) {
		unsigned int pop_count = BitList::BIT_COUNT_TABLE.lookUp( src );

		if ( pop_count > threshold ) {

			mNodes[nod++] = src;

			mCount[src] = getNodeCount( pop_count, threshold + pop_count - depth );

			mMap[src] = new unsigned int[ mCount[src] ];

			for ( unsigned int trg = 0, pos = 0; trg < size; trg++ ) {
				if ( BitList::BIT_COUNT_TABLE.lookUp( trg ) > threshold && ( src | trg ) == size - 1 ) {
					mMap[src][pos++] = trg;
				}
			}
		} 
	}
}

PatternGraph::~PatternGraph () {
	delete[] mMap;
	delete[] mCount;
	delete[] mNodes;
}

#if defined _LP64 || defined __LP64__
unsigned long PatternGraph::fac ( const unsigned char n ) const {
	unsigned long fac = 1;			
#else
unsigned long long PatternGraph::fac ( const unsigned char n ) const {
	unsigned long long fac = 1;
#endif // _LP64 || defined __LP64__

	for ( unsigned char i = 2; i <= n; i++ ) {
		fac *= i;
	} 
	return fac;
}

unsigned int PatternGraph::getNodeCount ( const unsigned int depth, const unsigned int threshold ) const {
	unsigned int count = 0;
			
	for ( unsigned int k = threshold + 1; k <= depth; k++ ) {
		count += ( fac( depth )/ fac( depth - k )) / fac( k );
	}
	return count;
}
