/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_POPULATION_INIT_VISITOR__
#define __DD_POPULATION_INIT_VISITOR__

#include "DDRayRefVisitor.h"
#include "DDInterval.h"

namespace addibit {

    class PopulationInitVisitor : public RayRefVisitor {

        private:
        Interval & mRangeS1;
        Interval & mRangeS2;
        Interval & mRangeDiff;

        protected:
        
        bool hasFinished () const {
            return true;
        }
    
        public:
        PopulationInitVisitor ( Interval & s_1, Interval & s_2, Interval & diff ) :
            RayRefVisitor(),
            mRangeS1( s_1 ),
            mRangeS2( s_2 ),
            mRangeDiff( diff ) {}

        ~PopulationInitVisitor () {}

        void onStart () {
            mRangeS1.reset();
            mRangeS2.reset();
            mRangeDiff.reset();
        }

        void onVisit ( const struct RayReference & ref ) {
            mRangeS1.update( ref.s_1 );
            mRangeS2.update( ref.s_2 );
            mRangeDiff.update( ref.s_2 - ref.s_1 );
        }
    };
}

#endif // __DD_POPULATION_INIT_VISITOR__
