/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_POPULATION_SPLIT_VISITOR__
#define __DD_POPULATION_SPLIT_VISITOR__

#include "DDBitSplitVisitor.h"
#include "DDRayRefPopulationMapping.h"
#include "DDContainerPopulationMapping.h"

namespace addibit {

    class PopulationSplitVisitor : public BitSplitVisitor {

        protected:
        RayRefPopulationMapping mRayRefMapping;
        ContainerPopulationMapping mContainerMapping;

        void registerContainer ( const unsigned int ) {
            mContainerMapping.insert_empty();
        }
        
        void registerReference ( const struct RayReference & ref,
                                 const unsigned int containerId ) {
            mContainerMapping.update( containerId, mRayRefMapping[ ref.id ] );
        }        
        
        unsigned int getTargetContainer ( const RayReference & ref ) {
            const unsigned int c_index = mRayRefMapping[ ref.id ];
            registerReference( ref, set( c_index ).id() );
            return c_index;
        }        

        public:
        PopulationSplitVisitor ( const unsigned int depth,
                                 const unsigned int capacity,
                                 const BitSelectStrategy & strategy,
                                 const unsigned int lbits,
                                 const unsigned int mbits ) :
            BitSplitVisitor( depth, capacity, strategy.getBitCount() - mbits ),
            mRayRefMapping( strategy, lbits, mbits ),
            mContainerMapping() {}

        ~PopulationSplitVisitor () {}

        void init ( const RayReferenceContainer & container ) {
            BitSplitVisitor::init( container );
            mRayRefMapping.init( container.size() );
            container.iterate( mRayRefMapping );    
        }
        
        bool can_proceed ( const unsigned int ) const {       
            return false;
        }           

        void refineCondition ( const uint_fast32_t index,
                               PopulationCondition & cond ) const {

      /*      BitSplitVisitor::refineCondition( index, edge_dim, cond );
            
            const unsigned int container_id = set( index ).id();
            
            if ( !mContainerMapping[ container_id ].empty() ) {
                const unsigned int pos = mContainerMapping[ container_id ].ub();
            
                cond.init( mRayRefMapping.getConditionalLimit(), mRayRefMapping.getFragment( pos ) );
            } */
        }
    };
}

#endif // __DD_POPULATION_SPLIT_VISITOR__
