/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_REF_BINARY_FRAGMENTATION_MAPPING__
#define __DD_RAY_REF_BINARY_FRAGMENTATION_MAPPING__

#include "DDRayRefVisitor.h"

namespace addibit {

    class RayRefBinaryFragmentationMapping : public RayRefVisitor {
    
        private:
        const unsigned int mBlocks;
        BitList * mRefMapping;
        
        public:
        RayRefBinaryFragmentationMapping ( const unsigned int blocks ) :
            mBlocks( blocks ) {}
        
        ~RayRefBinaryFragmentationMapping () {
            delete[] mRefMapping;
        }
        
        void init ( const unsigned int rcount ) {
            mRefMapping = new BitList[ rcount ];
        }        
        
        void onVisit ( const RayReference & ref ) {
            const BitList & m = ref.activeSet;
            mRefMapping[ ref.id ].resize( mBlocks );
            
            for ( unsigned int i = 0; i < mBlocks; i++ ) {
#if defined _LP64 || defined __LP64__
                mRefMapping[ ref.id ][ i ] = ((( 0xAAAAAAAAAAAAAAAA & m[i] ) & ~( m[i] << 1 )) ^ m[i] ) | (( 0xAAAAAAAAAAAAAAAA & m[i] ) >> 1 );
#else
				mRefMapping[ ref.id ][ i ] = ((( 0xAAAAAAAA & m[i] ) & ~( m[i] << 1 )) ^ m[i] ) | (( 0xAAAAAAAA & m[i] ) >> 1 );
#endif // defined _LP64 || defined __LP64__
            }
        }
        
        const BitList & operator[] ( const unsigned int id ) const {
            return mRefMapping[ id ];
        }
    };
}

#endif /* __DD_RAY_REF_BINARY_FRAGMENTATION_MAPPING__ */
