/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_REF_FRAGMENTATION_MAPPING__
#define __DD_RAY_REF_FRAGMENTATION_MAPPING__

#include "DDRayRefVisitor.h"
#include "DDFragmentationData.h"
#include "DDFragmentationTransformation.h"

namespace addibit {

    class RayRefFragmentationMapping : public RayRefVisitor {

        private:
        unsigned int mRefCount;

        const FragmentationTransformation & mTransformation;

        FragmentationData * __ref_mapping;

        public:
        RayRefFragmentationMapping ( const FragmentationTransformation & ftransformation ) :
            mRefCount( 0 ),       
            mTransformation( ftransformation ),
            __ref_mapping( NULL ) {}

        ~RayRefFragmentationMapping ();

        void init ( const uint_fast32_t rcount ) {
            mRefCount = rcount;
            __ref_mapping = new FragmentationData[ rcount ];
        }

        void onVisit ( const RayReference & ref ) {
             __ref_mapping[ ref.id ] = mTransformation.convert( ref.activeSet );
        }

        const FragmentationData & operator[] ( const uint_fast16_t id ) const {
            return __ref_mapping[ id ];
        }
    };
}

#endif /* __DD_RAY_REF_FRAGMENTATION_MAPPING__ */
