/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_REFERENCE_OVERAPPROXIMATION__
#define __DD_RAY_REFERENCE_OVERAPPROXIMATION__

#include "DDBitList.h"

namespace addibit {

    class RayReferenceOverapproximation {
    
        private:
        const BitList mUnionPattern;
        const BitList mCutPattern;
        const unsigned int mPopCnt;
        
        public:
        RayReferenceOverapproximation () :
            mUnionPattern( __BIT_LIST_ALLOC_SPACE__, ULONG_MAX ),
            mCutPattern( __BIT_LIST_ALLOC_SPACE__, ULONG_MAX ),
            mPopCnt( USHRT_MAX ) {}        
        
        RayReferenceOverapproximation ( const BitList & union_map ) :
            mUnionPattern( union_map ),
            mCutPattern( __BIT_LIST_ALLOC_SPACE__, ULONG_MAX ),
            mPopCnt( USHRT_MAX ) {}
                    
        RayReferenceOverapproximation ( const BitList & union_map,
                                        const BitList & cut_map ) :
            mUnionPattern( union_map ),
            mCutPattern( cut_map ),
            mPopCnt( USHRT_MAX ) {}
            
        RayReferenceOverapproximation ( const BitList & union_map,
                                        const BitList & cut_map,
                                        const unsigned int popcnt ) :
            mUnionPattern( union_map ),
            mCutPattern( cut_map ),
            mPopCnt( popcnt ) {}            
            
        const BitList & union_map () const {
            return mUnionPattern;
        }
        
        const BitList & cut_map () const {
            return mCutPattern;
        }
        
        unsigned int pop_cnt () const {
            return mPopCnt;
        }
        
        const std::string toString () const;
    };
}

#endif /* __DD_RAY_REFERENCE_OVERAPPROXIMATION__ */
