/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_ROOT_TREE_NODE__
#define __DD_ROOT_TREE_NODE__

#ifdef __DD_RECURSIVE_QUERY__
#include "DDAbstractIntermediateTreeNode.h"
#else
#include "DDConditionedTreeNode.h"
#endif

namespace addibit {
    
#ifdef __DD_RECURSIVE_QUERY__
    template<class TRANSFORMER>
	class RootTreeNode : public AbstractIntermediateTreeNode {
#else
    template<class TRANSFORMER, class COND_TYPE>
    class RootTreeNode : public TreeNode {
#endif 
	    private:
	    const TRANSFORMER * const mTransformation;
#ifndef __DD_RECURSIVE_QUERY__
        const ConditionedTreeNode * __entry_point;
        COND_TYPE mCondition;
#endif
	
	    public:
#ifdef __DD_RECURSIVE_QUERY__
	    RootTreeNode ( const unsigned int children,
	                   const TRANSFORMER * const trans ) :
	        AbstractIntermediateTreeNode( children ),
	        mTransformation( trans ) {}
#else
	    RootTreeNode ( const TRANSFORMER * const trans ) :
	        mTransformation( trans ) {}     
#endif   
     
		void narrow ( EdgeCandidatesContainer & container ) const {
                          
#ifdef __DD_RECURSIVE_QUERY__
	        AbstractIntermediateTreeNode::narrow( mTransformation->execute( current ), candidates );
#else
            mTransformation->apply( container.candidate(), container.stack() );
            __entry_point->narrow<COND_TYPE>( container );
#endif
        }    
        
		bool verify ( EdgeCandidate & candidate ) const {
#ifdef __DD_RECURSIVE_QUERY__
            return AbstractIntermediateTreeNode::verify( mTransformation->execute( candidate ) );
#else
            mTransformation->apply( candidate );
            return __entry_point->verify<COND_TYPE>( candidate );
#endif
        }      
        
        uint_fast16_t getNarrowingCandidateSize () const {
            return mTransformation->getNarrowingCandidateSize();
        }  
        
#ifndef __DD_RECURSIVE_QUERY__
        uint_fast64_t getRaysCount () const {
            const ConditionedTreeNode * __current = __entry_point;
            uint_fast64_t result = 0;
            
            do {
                result += __current->mArray.entries();
                __current = __current->on_cond_true();
            } while ( __current != NULL );

            return result;
        }

        const ConditionedTreeNode * entry_point () const {
            return __entry_point;
        }
    
        void set_entry_point ( const ConditionedTreeNode * entry_point ) {
            __entry_point = entry_point;
        }

        const std::string toString () const {
            return std::string( "ROOT" );
        }
        
        const std::string toGraphVizString ( const uint_fast16_t ) const {
            std::stringstream result;
            
    #ifdef __OUTPUT_TREES__
            const ConditionedTreeNode * __current = __entry_point;
            uint32_t trans_id = 1;
            result << "p" << static_cast<const void*>( __entry_point->__parent ) << " [shape=rectangle,fontsize=8,label=\"" << toString() << "\"];\n";

            do {
                result << "p" << static_cast<const void*>( __current ) << " [shape=rectangle,fontsize=8,label=\"" << __current->__label;
                if ( !__current->mArray.empty() ) {
                    result << ", |v|=" << __current->mArray.entries();
                }
                result << "\"];\n";
                result << "p" << static_cast<const void*>( __current->__parent ) << "->" << "p" << static_cast<const void*>( __current ) << " [label=" << ( trans_id++ ) << "];\n";
                 
                __current = __current->on_cond_true();
                
            } while ( __current != NULL );
    #endif

            return result.str();
        }         
#endif
	};
}

#endif /* __DD_ROOT_TREE_NODE__ */
