/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_TERMINAL_CACHE_ENTRY_NODE__
#define __DD_TERMINAL_CACHE_ENTRY_NODE__

#include <deque>
#include <vector>

#include "DDCacheEntryNode.h"

namespace addibit {

    template <class RAY_TYPE, class CACHE_ENTRY_TYPE>
    class TerminalCacheEntryNode : public CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> {
    
        private:
        std::deque<std::shared_ptr<const RAY_TYPE> > & mContainer;
        std::vector<uint_fast16_t> mStrictInequalities;
        
        public:
        TerminalCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                 std::deque<std::shared_ptr<const RAY_TYPE > > & container ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data ),
            mContainer( container ) {}      
            
        TerminalCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                 std::deque<std::shared_ptr<const RAY_TYPE > > & container,
                                 const uint_fast16_t id ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data, id ),
            mContainer( container ) {}               
            
        TerminalCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                 std::deque<std::shared_ptr<const RAY_TYPE > > & container,
                                 const std::vector<uint_fast16_t> & strict_inequalities ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data ),
            mContainer( container ),
            mStrictInequalities( strict_inequalities ) {}
            
        TerminalCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                 std::deque<std::shared_ptr<const RAY_TYPE > > & container,
                                 const std::vector<uint_fast16_t> & strict_inequalities,
                                 const uint_fast16_t id ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data, id ),
            mContainer( container ),
            mStrictInequalities( strict_inequalities ) {}          
            
        TerminalCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                 std::deque<std::shared_ptr<const RAY_TYPE > > & container,
                                 const std::vector<uint_fast16_t> & strict_inequalities,
                                 const uint_fast16_t id,
                                 const uint_fast16_t thread_id ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data, id, thread_id ),
            mContainer( container ),
            mStrictInequalities( strict_inequalities ) {}                   
           
        ~TerminalCacheEntryNode () {}
        
        using CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>::data;
        using CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>::id;
        using CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>::thread_id;
        
        bool push ( std::shared_ptr<const RAY_TYPE > ray ) {    
            
            for ( auto plane : mStrictInequalities ) {
		        if ( !ray->isInside( plane ) ) {
		            return false;
		        }
		    }
		    mContainer.push_back( ray );
            return true;
		}
		
		void insert ( CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> *, const uint_fast16_t ) {
		    DD_OUTPUT( "[ ERROR ] Inserting a new entry at a terminal node!" )
		}
        
        const std::string toString () const {
            std::stringstream result;
            
            result << CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>::toString() << ", strict = [ ";
            for ( auto inequality : mStrictInequalities ) {
                result << inequality << " ";
            }
            result << "]";
            
            return result.str();
        }        
    };
}

#endif /* __DD_TERMINAL_CACHE_ENTRY_NODE__ */
