/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_TERNARY_VANTAGE_POINT_VISITOR__
#define __DD_TERNARY_VANTAGE_POINT_VISITOR__

#include "DDVantagePointVisitor.h"

#define TERNARY_VP_SUBSET       0
#define TERNARY_VP_INV_SUBSET   1
#define TERNARY_VP_SUPERSET     2

namespace addibit {

    class TernaryVantagePointVisitor : public VantagePointVisitor {

        private:
        unsigned int getRangeIndex ( const RayReference & ref ) {
            unsigned int range_index = 0;
            const unsigned int dist = mVantagePoint.getCommonPopCnt( ref.activeSet );

            while ( mDistance[ range_index ] <= dist ) {
                range_index++;
            }

            return range_index;
        }

        void initDistBorders () {
            unsigned int distance_range_index = 0;
            unsigned int current = 0;

            for ( unsigned int set_index = 0; set_index <= mDistStats.size(); set_index++ ) {
                current += mDistStats[ set_index ];
                mDistance[ distance_range_index ]++;

                if ( current > mTotal * ( ( 1.0 * ( distance_range_index + 1 ) / getSplitFactor())) ) {
                    mDistance[ distance_range_index + 1 ] = mDistance[ distance_range_index ];
                    distance_range_index++;
                }
            }
        }

        protected:
        std::vector<unsigned int> mDistStats;
        unsigned int mTotal;

        unsigned int * mDistance;

        Interval * mDistanceRange;    
        Interval * mPopCntRange;

        void onInit ( const RayReference & ref ) {
            const unsigned int dist = mVantagePoint.getCommonPopCnt( ref.activeSet );

            mDistStats[ dist ]++;
            mTotal++;
        }

        void onSave ( const RayReference & ref ) {
            const unsigned int range_index = getRangeIndex( ref );

            set( range_index ).insert( ref );
            mDistanceRange[ range_index ].update( mVantagePoint.getCommonPopCnt( ref.activeSet ) );
            mPopCntRange[ range_index ].update( ref.degeneracy );
        }

        public:
        TernaryVantagePointVisitor ( const unsigned int depth,
                                    const unsigned int capacity ) :
            VantagePointVisitor( depth, capacity, 3 ) {

            mDistanceRange = new Interval[ 3 ];
            mPopCntRange = new Interval[ 3 ];
            mDistance = new unsigned int[ 3 ];
        }

        TernaryVantagePointVisitor ( const unsigned int depth,
                              const unsigned int capacity, 
                              const BitList & point ) :
            VantagePointVisitor( depth, capacity, 3, point ) {

            mDistanceRange = new Interval[ 3 ];
            mPopCntRange = new Interval[ 3 ];
            mDistance = new unsigned int[ 3 ];
        }

        ~TernaryVantagePointVisitor () {
            delete[] mDistance;
            delete[] mDistanceRange;
            delete[] mPopCntRange;
        }

        virtual void before ( const RayReferenceContainer & container ) {
            VantagePointVisitor::before( container );
   
            mInitRound = true;

            mDistStats.clear();
            mTotal = 0;
        }

        virtual void onStart () {
            VantagePointVisitor::onStart();

            if ( !mInitRound ) {

                for ( unsigned int i = 0; i < getSplitFactor(); i++ ) {
                    mDistance[ i ] = 0;
                    mDistanceRange[ i ].reset();
                    mPopCntRange[ i ].reset();
                }

                initDistBorders(); 
            }
        }

        const TreeNodeCondition * createNarrowingCondition ( const unsigned int index,
                                                                const unsigned int bits ) const {

            return new VantagePointTreeNodeCondition( set( index ).union_map(), bits, mVantagePoint, mDistanceRange[ index ], mPopCntRange[ index ], false );
        }

        void refineCondition ( const uint_fast32_t index,
                               const uint_fast16_t edge_dim,
                               VantagePointTreeNodeCondition & cond ) const {

            BitSplitVisitor::refineCondition( index, edge_dim, cond );
        
            cond.init( mVantagePoint, mDistanceRange[ index ], mPopCntRange[ index ], false );
        }

        bool supportsExtendedCondition () {
            return true;    
        }
    };
}

#endif // __DD_VANTAGE_POINT_VISITOR__
