/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_TREE_FACTORY__
#define __DD_TREE_FACTORY__

#include "DDOptions.h"
#include "DDAbstractTreeCreator.h"
#include "DDBitSelectStrategy.h"

namespace addibit {

    class TreeFactory {

        private:
        const uint_fast16_t mPlane;
        
        AbstractTreeCreator * getBitPatternTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
        AbstractTreeCreator * getExtendedBitPatternTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
        AbstractTreeCreator * getVantagePointTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
        AbstractTreeCreator * getFragmentedBitPatternTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
        AbstractTreeCreator * getPopulationTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
        AbstractTreeCreator * getFragmentationTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;

        inline uint_fast16_t plane () const {
            return mPlane;
        }

        public:
        TreeFactory ( const uint_fast16_t plane ) :
            mPlane( plane ) {}
            
        AbstractTreeCreator * getTreeCreator ( const TreeOptions &, const BitSelectStrategy & ) const;
    };
}

#endif /* __DD_TREE_FACTORY__ */
