/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_TREE_NODE__
#define __DD_TREE_NODE__

#include <vector>
#include <deque>

#include "DDLogging.h"

#include "DDTweaks.h"

#include "DDRayReference.h"
#include "DDTreeNodeCondition.h"

#include <string>
#include <fstream>

namespace addibit {

	class TreeNode {
		
		public:                    
        virtual ~TreeNode () {}
        
        virtual void narrow ( EdgeCandidatesContainer & ) const = 0;   
        virtual bool verify ( EdgeCandidate & ) const = 0;

        virtual uint_fast64_t getRaysCount () const = 0;

#ifdef __DD_RECURSIVE_QUERY__
        virtual uint_fast32_t getSubNodesCount () const = 0;

        virtual bool isEmpty () const = 0;
#endif

        void toFile ( const std::string &, const uint_fast16_t ) const;
        void toFile ( const char *, const uint_fast16_t, const uint_fast16_t ) const;

        virtual const std::string toString () const;
        virtual const std::string toGraphVizString ( const uint_fast16_t ) const = 0;

	};
}

#endif // __DD_TREE_NODE__
