/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_VANTAGE_POINT_TRANSFORMATION__
#define __DD_VANTAGE_POINT_TRANSFORMATION__

#include "DDBitSelectStrategy.h"
#include "DDNarrowingCandidate.h"

namespace addibit {
    
    class VantagePointTransformation {
    
        private:
        const uint_fast16_t __narrowing_candidate_size;
        const uint_fast16_t __stack_size;
        const BitList __static_vp;
            
        inline void apply ( const __BIT_LIST_RAW_DATA_TYPE__ * const data,
                            const uint16_t population,
                            AdjacencyTestStack * const stack ) const {
            
            stack->reset();
            VantagePointNodeData * aux_data = reinterpret_cast<VantagePointNodeData *>( stack->push( 2 ) );
            aux_data->__vec[0] = __static_vp.getCommonPopCnt( data );
            aux_data->__vec[1] = population - aux_data->__vec[0];
            aux_data[1] = aux_data[0];            
        }
            
        public:
        VantagePointTransformation ( const uint_fast16_t depth,
                                     const BitSelectStrategy & strategy ) :
            __narrowing_candidate_size( sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) * ( __DD_BLOCK_COUNT__ + 2 * ( depth + 1 ) ) ),
            __stack_size( 2 * ( depth + 1 ) * sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) ),
            __static_vp( BitSelector( strategy ).getFirstNBits( GLOBAL_EDGE_DIM ) ) {}
        
        inline void apply ( const NarrowingCandidate * cand, AdjacencyTestStack * const stack ) const { 
            apply( cand->data(), cand->population(), stack );
            
        /*    VantagePointNodeData * aux_data = reinterpret_cast<VantagePointNodeData *>( stack->push( 2 ) );
            aux_data->__vec[0] = __static_vp.getCommonPopCnt( cand->data() );
            aux_data->__vec[1] = cand->population() - aux_data->__vec[0];
            aux_data[1] = aux_data[0]; */
        }
        
        inline void apply ( EdgeCandidate & cand ) const { 
            apply( cand.adj_cand().data(), cand.population(), cand.stack() );
        }
        
        uint_fast16_t getNarrowingCandidateSize () const {
            return __narrowing_candidate_size;
        }
        
        uint_fast16_t getAdjacencyCandidateSize () const {
            return sizeof( AdjacencyCandidate ) + sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) * ( __DD_BLOCK_COUNT__ - 1 );
        }  
        
        uint_fast16_t getAdjacencyStackSize () const {
            return __stack_size;
        }
        
        const BitList & static_vp () const {
            return __static_vp;
        }
    };
}

#endif /* __DD_VANTAGE_POINT_TRANSFORMATION__ */
