/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_VER_QUERY_BITS_NEUTRALIZATION_STRATEGY__
#define __DD_VER_QUERY_BITS_NEUTRALIZATION_STRATEGY__

#include <vector>

#include "DDDifferentialBitStrategy.h"

namespace addibit {

    class VerQueryBitsNeutralizationStrategy : public DifferentialBitStrategy {

        private:
        const double_t * __ver_stats;

        protected:
        
        double_t getProbabilityCoeff ( const uint_fast16_t index,
                                       const double_t list_probability,
                                       const double_t tree_probability ) const {

            return pow( __ver_stats[ index ] - 1, 2 ) * pow( std::min( list_probability, tree_probability ), 2 ) ;
        }
        
        public:
        VerQueryBitsNeutralizationStrategy ( const std::vector<unsigned long> & ver_container_stats,
                                             const uint64_t ver_elements,
                                             const std::vector<unsigned long> & list_container_stats,
                                             const uint64_t list_elements,
                                             const std::vector<unsigned long> & tree_container_stats,
                                             const uint64_t tree_elements,
                                             const uint_fast16_t plane ) :
            DifferentialBitStrategy( list_container_stats, list_elements, tree_container_stats, tree_elements, plane ),
            __ver_stats( calc_probability( ver_container_stats, ver_elements ) ) {
                
          /*      for ( int i = 0; i < plane; ++i ) {
                    DD_OUTPUT( std::fixed << i << " : ver(" << ver_elements << ")=" << __ver_stats[i] << ", tree=" << __tree_probability[i] <<  ", list=" << __probability[i] )    
                } */
            }
            
        ~VerQueryBitsNeutralizationStrategy () {
            delete[] __ver_stats;
        }
        
    };
}

#endif // __DD_VER_QUERY_BITS_NEUTRALIZATION_STRATEGY__
