
#include <stdlib.h>
#include <vector>
#include <algorithm>
#include <ctime>
#include <cstdlib> 
#include <iostream>
#include <sstream>
#include <fstream>

std::vector<long> getRandoms( unsigned long count, long lb, long ub, bool unique ){
    std::vector<long> randoms;
    unsigned long generated = 0;
    long dist;

    if ( lb < 0 && ub > 0 ) {
        dist = abs(lb) + ub;
    } else {
        dist = abs(ub - lb);
    }

    while ( generated < count ) {
        long current = dist ? ((rand() % dist) + lb) : 0;
        if ( !unique || std::find( randoms.begin(), randoms.end(), current ) == randoms.end() ){
            randoms.push_back( current );
            generated++;
        }
    }

	return randoms;
}

void swap ( std::vector<long> & row_index, unsigned long dim, bool rotate = false ) {
    
    long current;
    
    do {
        current = rand() % dim;
    } while ( current == 0 || std::find( row_index.begin(), row_index.end(), current ) != row_index.end() );

    if ( rotate ) {
        row_index.insert( row_index.begin(), row_index.back() );
        row_index.pop_back(); 
    } else {
        row_index.pop_back();
        row_index.insert( row_index.begin(), current );
    }
}

std::vector<std::vector<long> > getInitialMatrix( unsigned long dim ) {
    std::vector<std::vector<long> > matrix;

    for ( unsigned long i = 0; i < dim; i++ ) {
        std::vector<long> row( dim, 0 );

        row[0] = 1;
        for ( unsigned long j = 1; j <= i; j++ ) {
            row[ j ] = 2048;
        }
        matrix.push_back( row );
    }

    return matrix;
}

bool check ( const std::vector<std::vector<long> > & matrix, const std::vector<long> & row ) {
    
    for ( auto & m_row: matrix ) {
        if ( equal( row.begin(), row.end(), m_row.begin()) ) {
            return false;
        }
    }
    
    return true;
}

void dump ( unsigned long rows, unsigned long dim, unsigned long count, long trunc ) {

    std::vector<std::vector<long> > matrix = getInitialMatrix( dim );
	std::vector<long> row_index_prev;

    std::vector<long> row_index = getRandoms( count, 1, dim, true );
   // std::vector<long> coeffs = getRandoms( count, -bound_1, bound_1, false );

    for ( unsigned long r = dim; r < rows; r++ ) {
        
   //     std::vector<long> row_index = getRandoms( count, r - dim, r - 1, true );
   //     std::vector<long> coeffs = getRandoms( count, -bound_1, bound_1, false );
		std::vector<long> row( dim, 0 );

        for ( unsigned int rindex = 0; rindex < dim; rindex++ ) {
            row[ rindex ] = matrix[ row_index[ 0 ] ][ rindex ];
        } 
        for ( unsigned long r_index = 1; r_index < row_index.size(); r_index++ ) {
            long bindex = row_index[ r_index ];
        //    long coeff = coeffs[ r_index ] ; // coeffs.back();
            
            for ( unsigned int rindex = 0; rindex < dim; rindex++ ) {
                row[ rindex ] = row[ rindex ] - matrix[ bindex ][ rindex ]/2;
            } 
        //    coeffs.pop_back();
        } 

        row[0] = 1;
        if ( check( matrix, row ) ) {
            matrix.push_back( row );
            swap( row_index, dim, ( r % trunc ) != 0 );
        } else {
            swap( row_index, dim, false );
            --r;
            continue;
        }
    }
    
    std::stringstream filename;
    filename << "random_" << count << "_" << dim << "_" << rows << ".ine"; 
    std::fstream ine_file( filename.str().c_str(), std::fstream::out );        

    ine_file << "H-representation" << std::endl
             << "begin" << std::endl
             << rows << " " << dim << " integer" << std::endl;
    
    for ( unsigned int rindex = 0; rindex < rows; rindex++ ) {
        for ( unsigned int cindex = 0; cindex < dim - 1; cindex++ ) {
            ine_file << matrix[ rindex ][ cindex ] << " ";
        }
        ine_file << matrix[ rindex ][ dim - 1 ] << std::endl;
    }
    
    ine_file << "end" << std::endl;
    ine_file.close();
}

int main ( int argc, const char** argv ) {

    srand(time(0));

    unsigned long instances = atoi( argv[1]);
    unsigned long dim = atoi( argv[2]);
	unsigned long count = atoi( argv[3]);    
    double row_to_dim_factor =  atof( argv[4]);    
    long trunc = atoi( argv[5]);

    for ( unsigned int pindex = 0; pindex < instances; pindex++ ) {
        dump( (unsigned long)( row_to_dim_factor * ( dim + pindex )), dim + pindex, count, trunc );
    }

	return 0;
}
